# -*- coding: utf-8 -*-
'''
Tests of utilities that support multiple masters in Salt Raet

'''
from __future__ import absolute_import
# pylint: skip-file
# pylint: disable=C0103
import sys
from salt.ext.six.moves import map
if sys.version_info < (2, 7):
    import unittest2 as unittest
else:
    import unittest

import os
import stat
import time
import tempfile
import shutil

from ioflo.aid.odicting import odict
from ioflo.aid.timing import Timer, StoreTimer
from ioflo.base import storing
from ioflo.base.consoling import getConsole
console = getConsole()

from raet import raeting

from salt.daemons import parse_hostname,  extract_masters


def setUpModule():
    console.reinit(verbosity=console.Wordage.concise)

def tearDownModule():
    pass

class BasicTestCase(unittest.TestCase):
    """"""

    def setUp(self):
        self.store = storing.Store(stamp=0.0)
        self.timer = StoreTimer(store=self.store, duration=1.0)
        self.port = 4506
        self.opts = dict(master_port=self.port)

    def tearDown(self):
        pass



    def testParseHostname(self):
        '''
        Test parsing hostname provided according to syntax for opts['master']
        '''
        console.terse("{0}\n".format(self.testParseHostname.__doc__))

        self.assertEquals(parse_hostname('localhost', self.port),
                                       ('localhost', 4506))
        self.assertEquals(parse_hostname('127.0.0.1', self.port),
                                       ('127.0.0.1', 4506))
        self.assertEquals(parse_hostname('10.0.2.100', self.port),
                                        ('10.0.2.100', 4506))
        self.assertEquals(parse_hostname('me.example.com', self.port),
                                        ('me.example.com', 4506))
        self.assertEquals(parse_hostname(
               '1.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.ip6.arpa',
                self.port),
                ('1.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.ip6.arpa',
                 4506))
        self.assertEquals(parse_hostname('fe80::1%lo0', self.port),
                                                ('fe80::1%lo0', 4506))

        self.assertEquals(parse_hostname('  localhost   ', self.port),
                                               ('localhost', 4506))
        self.assertEquals(parse_hostname('  127.0.0.1   ', self.port),
                                       ('127.0.0.1', 4506))
        self.assertEquals(parse_hostname('   10.0.2.100   ', self.port),
                                        ('10.0.2.100', 4506))
        self.assertEquals(parse_hostname('  me.example.com  ', self.port),
                                        ('me.example.com', 4506))
        self.assertEquals(parse_hostname(
               '  1.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.ip6.arpa   ',
                self.port),
                ('1.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.ip6.arpa',
                 4506))
        self.assertEquals(parse_hostname('  fe80::1%lo0  ', self.port),
                                                ('fe80::1%lo0', 4506))


        self.assertEquals(parse_hostname('localhost 4510', self.port),
                                               ('localhost', 4510))
        self.assertEquals(parse_hostname('127.0.0.1 4510', self.port),
                                       ('127.0.0.1', 4510))
        self.assertEquals(parse_hostname('10.0.2.100 4510', self.port),
                                        ('10.0.2.100', 4510))
        self.assertEquals(parse_hostname('me.example.com 4510', self.port),
                                        ('me.example.com', 4510))
        self.assertEquals(parse_hostname(
               '1.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.ip6.arpa 4510',
                self.port),
                ('1.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.ip6.arpa',
                 4510))
        self.assertEquals(parse_hostname('fe80::1%lo0 4510', self.port),
                                                ('fe80::1%lo0', 4510))


        self.assertEquals(parse_hostname('  localhost     4510 ', self.port),
                                               ('localhost', 4510))
        self.assertEquals(parse_hostname('   127.0.0.1    4510   ', self.port),
                                       ('127.0.0.1', 4510))
        self.assertEquals(parse_hostname('   10.0.2.100   4510   ', self.port),
                                        ('10.0.2.100', 4510))
        self.assertEquals(parse_hostname('   me.example.com    4510   ', self.port),
                                        ('me.example.com', 4510))
        self.assertEquals(parse_hostname(
               '   1.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.ip6.arpa   4510   ',
                self.port),
                ('1.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.ip6.arpa',
                 4510))
        self.assertEquals(parse_hostname('   fe80::1%lo0   4510   ', self.port),
                                                ('fe80::1%lo0', 4510))


        self.assertEquals(parse_hostname('localhost abcde', self.port), None)
        self.assertEquals(parse_hostname('127.0.0.1 a4510', self.port), None)
        self.assertEquals(parse_hostname(list([1, 2, 3]), self.port), None)
        self.assertEquals(parse_hostname(list(), self.port), None)
        self.assertEquals(parse_hostname(dict(a=1), self.port), None)
        self.assertEquals(parse_hostname(dict(), self.port), None)
        self.assertEquals(parse_hostname(4510, self.port), None)
        self.assertEquals(parse_hostname(('localhost', 4510), self.port), None)

        self.assertEquals(parse_hostname('localhost:4510', self.port),
                                               ('localhost', 4510))
        self.assertEquals(parse_hostname('127.0.0.1:4510', self.port),
                                       ('127.0.0.1', 4510))
        self.assertEquals(parse_hostname('10.0.2.100:4510', self.port),
                                        ('10.0.2.100', 4510))
        self.assertEquals(parse_hostname('me.example.com:4510', self.port),
                                        ('me.example.com', 4510))
        self.assertEquals(parse_hostname(
               '1.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.ip6.arpa:4510',
                self.port),
                ('1.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.ip6.arpa',
                 4510))
        self.assertEquals(parse_hostname('fe80::1%lo0:4510', self.port),
                                                      ('fe80::1%lo0:4510', 4506))
        self.assertEquals(parse_hostname('localhost::4510', self.port),
                                                       ('localhost::4510', 4506))


    def testExtractMastersSingle(self):
        '''
        Test extracting from master provided according to syntax for opts['master']
        '''
        console.terse("{0}\n".format(self.testExtractMastersSingle.__doc__))

        master = 'localhost'
        self.opts.update(master=master)
        self.assertEquals(extract_masters(self.opts),
                          [
                              dict(external=('localhost', 4506),
                                   internal=None),
                          ])

        master = '127.0.0.1'
        self.opts.update(master=master)
        self.assertEquals(extract_masters(self.opts),
                          [
                              dict(external=('127.0.0.1', 4506),
                                   internal=None),
                          ])

        master = 'localhost 4510'
        self.opts.update(master=master)
        self.assertEquals(extract_masters(self.opts),
                          [
                              dict(external=('localhost', 4510),
                                   internal=None),
                          ])

        master = '127.0.0.1 4510'
        self.opts.update(master=master)
        self.assertEquals(extract_masters(self.opts),
                          [
                              dict(external=('127.0.0.1', 4510),
                                   internal=None),
                          ])


        master = '10.0.2.23'
        self.opts.update(master=master)
        self.assertEquals(extract_masters(self.opts),
                          [
                              dict(external=('10.0.2.23', 4506),
                                   internal=None),
                          ])

        master = 'me.example.com'
        self.opts.update(master=master)
        self.assertEquals(extract_masters(self.opts),
                                  [
                                      dict(external=('me.example.com', 4506),
                                           internal=None),
                                  ])

        master = '10.0.2.23 4510'
        self.opts.update(master=master)
        self.assertEquals(extract_masters(self.opts),
                                  [
                                      dict(external=('10.0.2.23', 4510),
                                           internal=None),
                                  ])

        master = 'me.example.com 4510'
        self.opts.update(master=master)
        self.assertEquals(extract_masters(self.opts),
                                  [
                                      dict(external=('me.example.com', 4510),
                                           internal=None),
                                  ])

        master = dict(external='10.0.2.23 4510')
        self.opts.update(master=master)
        self.assertEquals(extract_masters(self.opts),
                                  [
                                      dict(external=('10.0.2.23', 4510),
                                           internal=None),
                                  ])

        master = dict(external='10.0.2.23 4510', internal='')
        self.opts.update(master=master)
        self.assertEquals(extract_masters(self.opts),
                                  [
                                      dict(external=('10.0.2.23', 4510),
                                           internal=None),
                                  ])

        master = dict(internal='10.0.2.23 4510')
        self.opts.update(master=master)
        self.assertEquals(extract_masters(self.opts),[])


    def testExtractMastersMultiple(self):
        '''
        Test extracting from master provided according to syntax for opts['master']
        '''
        console.terse("{0}\n".format(self.testExtractMastersMultiple.__doc__))

        master = [
                    'localhost',
                    '10.0.2.23',
                    'me.example.com'
                 ]
        self.opts.update(master=master)
        self.assertEquals(extract_masters(self.opts),
                          [
                            {
                                'external': ('localhost', 4506),
                                'internal': None
                            },
                            {
                                'external': ('10.0.2.23', 4506),
                                'internal': None
                            },
                            {
                                'external': ('me.example.com', 4506),
                                'internal': None
                            },
                          ])

        master = [
                    'localhost 4510',
                    '10.0.2.23 4510',
                    'me.example.com 4510'
                 ]
        self.opts.update(master=master)
        self.assertEquals(extract_masters(self.opts),
                          [
                            {
                                'external': ('localhost', 4510),
                                'internal': None
                            },
                            {
                                'external': ('10.0.2.23', 4510),
                                'internal': None
                            },
                            {
                                'external': ('me.example.com', 4510),
                                'internal': None
                            },
                          ])

        master = [
                    {
                        'external': 'localhost 4510',
                        'internal': '',
                    },
                    {
                        'external': 'me.example.com 4510',
                        'internal': '10.0.2.23 4510',
                    },
                    {
                        'external': 'you.example.com 4509',
                    }
                 ]
        self.opts.update(master=master)
        self.assertEquals(extract_masters(self.opts),
                          [
                            {
                                'external': ('localhost', 4510),
                                'internal': None
                            },
                            {
                                'external': ('me.example.com', 4510),
                                'internal': ('10.0.2.23', 4510)
                            },
                            {
                                'external': ('you.example.com', 4509),
                                'internal': None
                            },
                          ])


def runOne(test):
    '''
    Unittest Runner
    '''
    test = BasicTestCase(test)
    suite = unittest.TestSuite([test])
    unittest.TextTestRunner(verbosity=2).run(suite)

def runSome():
    '''
    Unittest runner
    '''
    tests =  []
    names = [
                'testParseHostname',
                'testExtractMastersSingle',
                'testExtractMastersMultiple',
            ]

    tests.extend(list(list(map(BasicTestCase, names))))

    suite = unittest.TestSuite(tests)
    unittest.TextTestRunner(verbosity=2).run(suite)

def runAll():
    '''
    Unittest runner
    '''
    suite = unittest.TestSuite()
    suite.addTest(unittest.TestLoader().loadTestsFromTestCase(BasicTestCase))

    unittest.TextTestRunner(verbosity=2).run(suite)

if __name__ == '__main__' and __package__ is None:

    #console.reinit(verbosity=console.Wordage.concise)

    #runAll() #run all unittests

    runSome()#only run some

    #runOne('testParseHostname')



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Jayesh Kariya <jayeshk@saltstack.com>`
'''

# Import Python Libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import TestCase, skipIf
from salttesting.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

from salttesting.helpers import ensure_in_syspath

ensure_in_syspath('../../')

# Import Salt Libs
from salt.modules import win_ip
from salt.exceptions import CommandExecutionError, SaltInvocationError

# Globals
win_ip.__salt__ = {}

ETHERNET_CONFIG = ('Configuration for interface "Ethernet"\n'
                   'DHCP enabled: Yes\n'
                   'IP Address: 1.2.3.74\n'
                   'Subnet Prefix: 1.2.3.0/24 (mask 255.255.255.0)\n'
                   'Default Gateway: 1.2.3.1\n'
                   'Gateway Metric: 0\n'
                   'InterfaceMetric: 20\n'
                   'DNS servers configured through DHCP: 1.2.3.4\n'
                   'Register with which suffix: Primary only\n'
                   'WINS servers configured through DHCP: None\n')

ETHERNET_ENABLE = ('Ethernet\n'
                   'Type: Dedicated\n'
                   'Administrative state: Enabled\n'
                   'Connect state: Connected')


@skipIf(NO_MOCK, NO_MOCK_REASON)
class WinShadowTestCase(TestCase):
    '''
    Test cases for salt.modules.win_ip
    '''
    # 'raw_interface_configs' function tests: 1

    def test_raw_interface_configs(self):
        '''
        Test if it return raw configs for all interfaces.
        '''
        mock_cmd = MagicMock(return_value=ETHERNET_CONFIG)
        with patch.dict(win_ip.__salt__, {'cmd.run': mock_cmd}):
            self.assertEqual(win_ip.raw_interface_configs(), ETHERNET_CONFIG)

    # 'get_all_interfaces' function tests: 1

    def test_get_all_interfaces(self):
        '''
        Test if it return configs for all interfaces.
        '''
        ret = {'Ethernet': {'DHCP enabled': 'Yes',
                            'DNS servers configured through DHCP': ['1.2.3.4'],
                            'Default Gateway': '1.2.3.1',
                            'Gateway Metric': '0',
                            'InterfaceMetric': '20',
                            'Register with which suffix': 'Primary only',
                            'WINS servers configured through DHCP': ['None'],
                            'ip_addrs': [{'IP Address': '1.2.3.74',
                                          'Netmask': '255.255.255.0',
                                          'Subnet': '1.2.3.0/24'}]}}

        mock_cmd = MagicMock(return_value=ETHERNET_CONFIG)
        with patch.dict(win_ip.__salt__, {'cmd.run': mock_cmd}):
            self.assertDictEqual(win_ip.get_all_interfaces(), ret)

    # 'get_interface' function tests: 1

    def test_get_interface(self):
        '''
        Test if it return the configuration of a network interface.
        '''
        ret = {'DHCP enabled': 'Yes',
               'DNS servers configured through DHCP': ['1.2.3.4'],
               'Default Gateway': '1.2.3.1', 'Gateway Metric': '0',
               'InterfaceMetric': '20',
               'Register with which suffix': 'Primary only',
               'WINS servers configured through DHCP': ['None'],
               'ip_addrs': [{'IP Address': '1.2.3.74',
                             'Netmask': '255.255.255.0',
                             'Subnet': '1.2.3.0/24'}]}

        mock_cmd = MagicMock(return_value=ETHERNET_CONFIG)
        with patch.dict(win_ip.__salt__, {'cmd.run': mock_cmd}):
            self.assertDictEqual(win_ip.get_interface('Ethernet'), ret)

    # 'is_enabled' function tests: 1

    def test_is_enabled(self):
        '''
        Test if it returns `True` if interface is enabled, otherwise `False`.
        '''
        mock_cmd = MagicMock(side_effect=[ETHERNET_ENABLE, ''])
        with patch.dict(win_ip.__salt__, {'cmd.run': mock_cmd}):
            self.assertTrue(win_ip.is_enabled('Ethernet'))
            self.assertRaises(CommandExecutionError, win_ip.is_enabled,
                              'Ethernet')

    # 'is_disabled' function tests: 1

    def test_is_disabled(self):
        '''
        Test if it returns `True` if interface is disabled, otherwise `False`.
        '''
        mock_cmd = MagicMock(return_value=ETHERNET_ENABLE)
        with patch.dict(win_ip.__salt__, {'cmd.run': mock_cmd}):
            self.assertFalse(win_ip.is_disabled('Ethernet'))

    # 'enable' function tests: 1

    def test_enable(self):
        '''
        Test if it enable an interface.
        '''
        mock_cmd = MagicMock(return_value=ETHERNET_ENABLE)
        with patch.dict(win_ip.__salt__, {'cmd.run': mock_cmd}):
            self.assertTrue(win_ip.enable('Ethernet'))

        mock_cmd = MagicMock(return_value='Connect state: Disconnected')
        with patch.dict(win_ip.__salt__, {'cmd.run': mock_cmd}):
            self.assertFalse(win_ip.enable('Ethernet'))

    # 'disable' function tests: 1

    def test_disable(self):
        '''
        Test if it disable an interface.
        '''
        mock_cmd = MagicMock(return_value=ETHERNET_ENABLE)
        with patch.dict(win_ip.__salt__, {'cmd.run': mock_cmd}):
            self.assertFalse(win_ip.disable('Ethernet'))

        mock_cmd = MagicMock(return_value='Connect state: Disconnected')
        with patch.dict(win_ip.__salt__, {'cmd.run': mock_cmd}):
            self.assertTrue(win_ip.disable('Ethernet'))

    # 'get_subnet_length' function tests: 1

    def test_get_subnet_length(self):
        '''
        Test if it disable an interface.
        '''
        self.assertEqual(win_ip.get_subnet_length('255.255.255.0'), 24)
        self.assertRaises(SaltInvocationError, win_ip.get_subnet_length,
                          '255.255.0')

    # 'set_static_ip' function tests: 1

    def test_set_static_ip(self):
        '''
        Test if it set static IP configuration on a Windows NIC.
        '''
        self.assertRaises(SaltInvocationError, win_ip.set_static_ip,
                          'Local Area Connection', '10.1.2/24')

        mock_cmd = MagicMock(return_value=ETHERNET_CONFIG)
        mock_all = MagicMock(return_value={'retcode': 1, 'stderr': 'Error'})
        with patch.dict(win_ip.__salt__, {'cmd.run': mock_cmd,
                                          'cmd.run_all': mock_all}):
            self.assertRaises(CommandExecutionError, win_ip.set_static_ip,
                              'Ethernet', '1.2.3.74/24', append=True)
            self.assertRaises(CommandExecutionError, win_ip.set_static_ip,
                              'Ethernet', '1.2.3.74/24')

        mock_all = MagicMock(return_value={'retcode': 0})
        with patch.dict(win_ip.__salt__, {'cmd.run': mock_cmd,
                                          'cmd.run_all': mock_all}):
            self.assertDictEqual(win_ip.set_static_ip('Local Area Connection',
                                                      '1.2.3.74/24'), {})
            self.assertDictEqual(win_ip.set_static_ip('Ethernet',
                                                      '1.2.3.74/24'),
                                 {'Address Info': {'IP Address': '1.2.3.74',
                                                   'Netmask': '255.255.255.0',
                                                   'Subnet': '1.2.3.0/24'}})

    # 'set_dhcp_ip' function tests: 1

    def test_set_dhcp_ip(self):
        '''
        Test if it set Windows NIC to get IP from DHCP.
        '''
        mock_cmd = MagicMock(return_value=ETHERNET_CONFIG)
        with patch.dict(win_ip.__salt__, {'cmd.run': mock_cmd}):
            self.assertDictEqual(win_ip.set_dhcp_ip('Ethernet'),
                                 {'DHCP enabled': 'Yes',
                                  'Interface': 'Ethernet'})

    # 'set_static_dns' function tests: 1

    def test_set_static_dns(self):
        '''
        Test if it set static DNS configuration on a Windows NIC.
        '''
        mock_cmd = MagicMock(return_value=ETHERNET_CONFIG)
        with patch.dict(win_ip.__salt__, {'cmd.run': mock_cmd}):
            self.assertDictEqual(win_ip.set_static_dns('Ethernet',
                                                       '192.168.1.252',
                                                       '192.168.1.253'),
                                 {'DNS Server': ('192.168.1.252',
                                                 '192.168.1.253'),
                                  'Interface': 'Ethernet'})

    # 'set_dhcp_dns' function tests: 1

    def test_set_dhcp_dns(self):
        '''
        Test if it set DNS source to DHCP on Windows.
        '''
        mock_cmd = MagicMock(return_value=ETHERNET_CONFIG)
        with patch.dict(win_ip.__salt__, {'cmd.run': mock_cmd}):
            self.assertDictEqual(win_ip.set_dhcp_dns('Ethernet'),
                                 {'DNS Server': 'DHCP',
                                  'Interface': 'Ethernet'})

    # 'set_dhcp_all' function tests: 1

    def test_set_dhcp_all(self):
        '''
        Test if it set both IP Address and DNS to DHCP.
        '''
        mock_cmd = MagicMock(return_value=ETHERNET_CONFIG)
        with patch.dict(win_ip.__salt__, {'cmd.run': mock_cmd}):
            self.assertDictEqual(win_ip.set_dhcp_all('Ethernet'),
                                 {'Interface': 'Ethernet', 'DNS Server': 'DHCP',
                                  'DHCP enabled': 'Yes'})

    # 'get_default_gateway' function tests: 1

    def test_get_default_gateway(self):
        '''
        Test if it set DNS source to DHCP on Windows.
        '''
        mock_cmd = MagicMock(return_value=ETHERNET_CONFIG)
        with patch.dict(win_ip.__salt__, {'cmd.run': mock_cmd}):
            self.assertEqual(win_ip.get_default_gateway(), '1.2.3.1')


if __name__ == '__main__':
    from integration import run_tests
    run_tests(WinShadowTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
Management of Zabbix host groups.

:codeauthor: Jiri Kotlin <jiri.kotlin@ultimum.io>


'''


def __virtual__():
    '''
    Only make these states available if Zabbix module is available.
    '''
    return 'zabbix.hostgroup_create' in __salt__


def present(name):
    '''
    Ensures that the host group exists, eventually creates new host group.

    .. versionadded:: 2016.3.0

    :param name: name of the host group
    :param _connection_user: Optional - zabbix user (can also be set in opts or pillar, see module's docstring)
    :param _connection_password: Optional - zabbix password (can also be set in opts or pillar, see module's docstring)
    :param _connection_url: Optional - url of zabbix frontend (can also be set in opts, pillar, see module's docstring)

    .. code-block:: yaml

        create_testing_host_group:
            zabbix_hostgroup.present:
                - name: 'My hostgroup name'


    '''
    ret = {'name': name, 'changes': {}, 'result': False, 'comment': ''}

    # Comment and change messages
    comment_hostgroup_created = 'Host group {0} created.'.format(name)
    comment_hostgroup_notcreated = 'Unable to create host group: {0}. '.format(name)
    comment_hostgroup_exists = 'Host group {0} already exists.'.format(name)
    changes_hostgroup_created = {name: {'old': 'Host group {0} does not exist.'.format(name),
                                        'new': 'Host group {0} created.'.format(name),
                                        }
                                 }

    hostgroup_exists = __salt__['zabbix.hostgroup_exists'](name)

    # Dry run, test=true mode
    if __opts__['test']:
        if hostgroup_exists:
            ret['result'] = True
            ret['comment'] = comment_hostgroup_exists
        else:
            ret['result'] = None
            ret['comment'] = comment_hostgroup_created
            ret['changes'] = changes_hostgroup_created
        return ret

    if hostgroup_exists:
        ret['result'] = True
        ret['comment'] = comment_hostgroup_exists
    else:
        hostgroup_create = __salt__['zabbix.hostgroup_create'](name)

        if 'error' not in hostgroup_create:
            ret['result'] = True
            ret['comment'] = comment_hostgroup_created
            ret['changes'] = changes_hostgroup_created
        else:
            ret['result'] = False
            ret['comment'] = comment_hostgroup_notcreated + str(hostgroup_create['error'])

    return ret


def absent(name):
    '''
    Ensures that the host group does not exist, eventually delete host group.

    .. versionadded:: 2016.3.0

    :param name: name of the host group
    :param _connection_user: Optional - zabbix user (can also be set in opts or pillar, see module's docstring)
    :param _connection_password: Optional - zabbix password (can also be set in opts or pillar, see module's docstring)
    :param _connection_url: Optional - url of zabbix frontend (can also be set in opts, pillar, see module's docstring)

    .. code-block:: yaml

        delete_testing_host_group:
            zabbix_hostgroup.absent:
                - name: 'My hostgroup name'

    '''

    ret = {'name': name, 'changes': {}, 'result': False, 'comment': ''}

    # Comment and change messages
    comment_hostgroup_deleted = 'Host group {0} deleted.'.format(name)
    comment_hostgroup_notdeleted = 'Unable to delete host group: {0}. '.format(name)
    comment_hostgroup_notexists = 'Host group {0} does not exist.'.format(name)
    changes_hostgroup_deleted = {name: {'old': 'Host group {0} exists.'.format(name),
                                        'new': 'Host group {0} deleted.'.format(name),
                                        }
                                 }

    hostgroup_exists = __salt__['zabbix.hostgroup_exists'](name)

    # Dry run, test=true mode
    if __opts__['test']:
        if not hostgroup_exists:
            ret['result'] = True
            ret['comment'] = comment_hostgroup_notexists
        else:
            ret['result'] = None
            ret['comment'] = comment_hostgroup_deleted
            ret['changes'] = changes_hostgroup_deleted
        return ret

    hostgroup_get = __salt__['zabbix.hostgroup_get'](name)

    if not hostgroup_get:
        ret['result'] = True
        ret['comment'] = comment_hostgroup_notexists
    else:
        try:
            groupid = hostgroup_get[0]['groupid']
            hostgroup_delete = __salt__['zabbix.hostgroup_delete'](groupid)
        except KeyError:
            hostgroup_delete = False

        if hostgroup_delete and 'error' not in hostgroup_delete:
            ret['result'] = True
            ret['comment'] = comment_hostgroup_deleted
            ret['changes'] = changes_hostgroup_deleted
        else:
            ret['result'] = False
            ret['comment'] = comment_hostgroup_notdeleted + str(hostgroup_delete['error'])

    return ret



# -*- coding: utf-8 -*-
'''
Utilities for managing kickstart

.. versionadded:: Beryllium
'''
from __future__ import absolute_import
import yaml
import shlex
import argparse  # pylint: disable=minimum-python-version
import salt.utils
from salt.ext.six.moves import range


def clean_args(args):
    '''
    Cleans up the args that weren't passed in
    '''
    for arg in args.keys():
        if not args[arg]:
            del args[arg]
    return args


def parse_auth(rule):
    '''
    Parses the auth/authconfig line
    '''
    parser = argparse.ArgumentParser()
    rules = shlex.split(rule)
    rules.pop(0)
    noargs = ('back', 'test', 'nostart', 'kickstart', 'probe', 'enablecache',
              'disablecache', 'disablenis', 'enableshadow', 'disableshadow',
              'enablemd5', 'disablemd5', 'enableldap', 'enableldapauth',
              'enableldaptls', 'disableldap', 'disableldapauth',
              'enablekrb5kdcdns', 'disablekrb5kdcdns', 'enablekrb5realmdns',
              'disablekrb5realmdns', 'disablekrb5', 'disablehe-siod',
              'enablesmbauth', 'disablesmbauth', 'enablewinbind',
              'enablewinbindauth', 'disablewinbind', 'disablewinbindauth',
              'enablewinbindusedefaultdomain', 'disablewinbindusedefaultdomain',
              'enablewins', 'disablewins')
    for arg in noargs:
        parser.add_argument('--{0}'.format(arg), dest=arg, action='store_true')

    parser.add_argument('--enablenis', dest='enablenis', action='store')
    parser.add_argument('--hesiodrhs', dest='hesiodrhs', action='store')
    parser.add_argument('--krb5adminserver', dest='krb5adminserver',
                        action='append')
    parser.add_argument('--krb5kdc', dest='krb5kdc', action='append')
    parser.add_argument('--ldapbasedn', dest='ldapbasedn', action='store')
    parser.add_argument('--ldapserver', dest='ldapserver', action='append')
    parser.add_argument('--nisserver', dest='nisserver', action='append')
    parser.add_argument('--passalgo', dest='passalgo', action='store')
    parser.add_argument('--smbidmapgid', dest='smbidmapgid', action='store')
    parser.add_argument('--smbidmapuid', dest='smbidmapuid', action='store')
    parser.add_argument('--smbrealm', dest='smbrealm', action='store')
    parser.add_argument('--smbsecurity', dest='smbsecurity', action='store',
                        choices=['user', 'server', 'domain', 'dns'])
    parser.add_argument('--smbservers', dest='smbservers', action='store')
    parser.add_argument('--smbworkgroup', dest='smbworkgroup', action='store')
    parser.add_argument('--winbindjoin', dest='winbindjoin', action='store')
    parser.add_argument('--winbindseparator', dest='winbindseparator',
                        action='store')
    parser.add_argument('--winbindtemplatehomedir',
                        dest='winbindtemplatehomedir', action='store')
    parser.add_argument('--winbindtemplateprimarygroup',
                        dest='winbindtemplateprimarygroup', action='store')
    parser.add_argument('--winbindtemplateshell', dest='winbindtemplateshell',
                        action='store')

    parser.add_argument('--enablekrb5', dest='enablekrb5', action='store_true')
    if '--enablekrb5' in rules:
        parser.add_argument('--krb5realm', dest='krb5realm', action='store',
                            required=True)
    parser.add_argument('--enablehesiod', dest='enablehesiod',
                        action='store_true')
    if '--enablehesiod' in rules:
        parser.add_argument('--hesiodlhs', dest='hesiodlhs', action='store',
                            required=True)

    args = clean_args(vars(parser.parse_args(rules)))
    parser = None
    return args


def parse_autopart(rule):
    '''
    Parse the autopart line
    '''
    parser = argparse.ArgumentParser()
    rules = shlex.split(rule)
    rules.pop(0)
    parser.add_argument('--type', dest='type', action='store')
    parser.add_argument('--encrypted', dest='encrypted', action='store_true')
    parser.add_argument('--passphrase', dest='passphrase', action='store')
    parser.add_argument('--escrowcert', dest='escrowcert', action='store')
    parser.add_argument('--backuppassphrase', dest='backuppassphrase',
                        action='store')

    args = clean_args(vars(parser.parse_args(rules)))
    parser = None
    return args


def parse_autostep(rule):
    '''
    Parse the autostep line
    '''
    parser = argparse.ArgumentParser()
    rules = shlex.split(rule)
    rules.pop(0)
    parser.add_argument('--autoscreenshot', dest='autoscreenshot',
                        action='store')

    args = clean_args(vars(parser.parse_args(rules)))
    parser = None
    return args


def parse_bootloader(rule):
    '''
    Parse the bootloader line
    '''
    parser = argparse.ArgumentParser()
    rules = shlex.split(rule)
    rules.pop(0)
    parser.add_argument('--append', dest='append', action='store')
    parser.add_argument('--driveorder', dest='driveorder', action='store')
    parser.add_argument('--location', dest='location', action='store')
    parser.add_argument('--password', dest='password', action='store')
    parser.add_argument('--md5pass', dest='md5pass', action='store')
    parser.add_argument('--upgrade', dest='upgrade', action='store_true')
    parser.add_argument('--timeout', dest='timeout', action='store')
    parser.add_argument('--boot-drive', dest='bootdrive', action='store')

    args = clean_args(vars(parser.parse_args(rules)))
    parser = None
    return args


def parse_btrfs(rule):
    '''
    Parse the btrfs line

    TODO: finish up the weird parsing on this one
    http://fedoraproject.org/wiki/Anaconda/Kickstart#btrfs
    '''
    parser = argparse.ArgumentParser()
    rules = shlex.split(rule)
    rules.pop(0)
    parser.add_argument('--name', dest='name', action='store')
    parser.add_argument('--data', dest='data', action='store')
    parser.add_argument('--metadata', dest='metadata', action='store')
    parser.add_argument('--label', dest='label', action='store')
    parser.add_argument('--noformat', dest='noformat', action='store_true')
    parser.add_argument('--useexisting', dest='useexisting',
                        action='store_true')
    parser.add_argument('--subvol', dest='subvol', action='store_true')

    args = clean_args(vars(parser.parse_args(rules)))
    parser = None
    return args


def parse_clearpart(rule):
    '''
    Parse the clearpart line
    '''
    parser = argparse.ArgumentParser()
    rules = shlex.split(rule)
    rules.pop(0)
    parser.add_argument('--all', dest='all', action='store_true')
    parser.add_argument('--drives', dest='drives', action='store')
    parser.add_argument('--init_label', dest='init_label', action='store_true')
    parser.add_argument('--linux', dest='linux', action='store_true')
    parser.add_argument('--none', dest='none', action='store_true')
    parser.add_argument('--initlabel', dest='init_label', action='store_true')
    parser.add_argument('--list', dest='list', action='store')

    args = clean_args(vars(parser.parse_args(rules)))
    parser = None
    return args


def parse_device(rule):
    '''
    Parse the device line
    '''
    parser = argparse.ArgumentParser()
    rules = shlex.split(rule)
    rules.pop(0)
    modulename = rules.pop(0)
    parser.add_argument('--opts', dest='opts', action='store')

    args = clean_args(vars(parser.parse_args(rules)))
    args['modulename'] = modulename
    parser = None
    return args


def parse_dmraid(rule):
    '''
    Parse the dmraid line
    '''
    parser = argparse.ArgumentParser()
    rules = shlex.split(rule)
    rules.pop(0)
    parser.add_argument('--name', dest='name', action='store')
    parser.add_argument('--dev', dest='dev', action='store')

    args = clean_args(vars(parser.parse_args(rules)))
    parser = None
    return args


def parse_driverdisk(rule):
    '''
    Parse the driverdisk line
    '''
    if '--' not in rule:
        return {'partition': rule}

    parser = argparse.ArgumentParser()
    rules = shlex.split(rule)
    rules.pop(0)
    parser.add_argument('--source', dest='source', action='store')
    parser.add_argument('--biospart', dest='biospart', action='store')

    args = clean_args(vars(parser.parse_args(rules)))
    parser = None
    return args


def parse_firewall(rule):
    '''
    Parse the firewall line
    '''
    parser = argparse.ArgumentParser()
    rules = shlex.split(rule)
    rules.pop(0)
    parser.add_argument('--enable', '--enabled', dest='enable',
                        action='store_true')
    parser.add_argument('--disable', '--disabled', dest='disable',
                        action='store_true')
    parser.add_argument('--port', dest='port', action='store')
    parser.add_argument('--service', dest='service', action='store')
    parser.add_argument('--ssh', dest='ssh', action='store_true')
    parser.add_argument('--smtp', dest='smtp', action='store_true')
    parser.add_argument('--http', dest='http', action='store_true')
    parser.add_argument('--ftp', dest='ftp', action='store_true')

    args = clean_args(vars(parser.parse_args(rules)))
    parser = None
    return args


def parse_firstboot(rule):
    '''
    Parse the firstboot line
    '''
    parser = argparse.ArgumentParser()
    rules = shlex.split(rule)
    rules.pop(0)
    parser.add_argument('--enable', '--enabled', dest='enable',
                        action='store_true')
    parser.add_argument('--disable', '--disabled', dest='disable',
                        action='store_true')
    parser.add_argument('--reconfig', dest='reconfig', action='store_true')

    args = clean_args(vars(parser.parse_args(rules)))
    parser = None
    return args


def parse_group(rule):
    '''
    Parse the group line
    '''
    parser = argparse.ArgumentParser()
    rules = shlex.split(rule)
    rules.pop(0)
    parser.add_argument('--name', dest='name', action='store')
    parser.add_argument('--gid', dest='gid', action='store')

    args = clean_args(vars(parser.parse_args(rules)))
    parser = None
    return args


def parse_harddrive(rule):
    '''
    Parse the harddrive line
    '''
    parser = argparse.ArgumentParser()
    rules = shlex.split(rule)
    rules.pop(0)
    parser.add_argument('--biospart', dest='biospart', action='store')
    parser.add_argument('--partition', dest='partition', action='store')
    parser.add_argument('--dir', dest='dir', action='store')

    args = clean_args(vars(parser.parse_args(rules)))
    parser = None
    return args


def parse_ignoredisk(rule):
    '''
    Parse the ignoredisk line
    '''
    parser = argparse.ArgumentParser()
    rules = shlex.split(rule)
    rules.pop(0)
    parser.add_argument('--drives', dest='drives', action='store')
    parser.add_argument('--only-use', dest='only-use', action='store')
    parser.add_argument('--interactive', dest='interactive',
                        action='store_true')

    args = clean_args(vars(parser.parse_args(rules)))
    parser = None
    return args


def parse_iscsi(rule):
    '''
    Parse the iscsi line
    '''
    parser = argparse.ArgumentParser()
    rules = shlex.split(rule)
    rules.pop(0)
    parser.add_argument('--ipaddr', dest='ipaddr', action='store')
    parser.add_argument('--port', dest='port', action='store')
    parser.add_argument('--target', dest='target', action='store')
    parser.add_argument('--iface', dest='iface', action='store')
    parser.add_argument('--user', dest='user', action='store')
    parser.add_argument('--password', dest='password', action='store')
    parser.add_argument('--reverse-user', dest='reverse-user', action='store')
    parser.add_argument('--reverse-password', dest='reverse-password',
                        action='store')

    args = clean_args(vars(parser.parse_args(rules)))
    parser = None
    return args


def parse_iscsiname(rule):
    '''
    Parse the iscsiname line
    '''
    parser = argparse.ArgumentParser()
    rules = shlex.split(rule)
    rules.pop(0)
    #parser.add_argument('iqn')

    args = clean_args(vars(parser.parse_args(rules)))
    parser = None
    return args


def parse_keyboard(rule):
    '''
    Parse the keyboard line
    '''
    parser = argparse.ArgumentParser()
    rules = shlex.split(rule)
    rules.pop(0)
    parser.add_argument('--vckeymap', dest='vckeymap', action='store')
    parser.add_argument('--xlayouts', dest='xlayouts', action='store')
    parser.add_argument('--switch', dest='switch', action='store')
    parser.add_argument('keyboard')

    args = clean_args(vars(parser.parse_args(rules)))

    if 'keyboard' in args and 'xlayouts' not in args:
        args['xlayouts'] = args['keyboard']

    parser = None
    return args


def parse_lang(rule):
    '''
    Parse the lang line
    '''
    parser = argparse.ArgumentParser()
    rules = shlex.split(rule)
    rules.pop(0)
    parser.add_argument('lang')

    args = clean_args(vars(parser.parse_args(rules)))
    parser = None
    return args


def parse_logvol(rule):
    '''
    Parse the logvol line
    '''
    parser = argparse.ArgumentParser()
    rules = shlex.split(rule)
    rules.pop(0)
    parser.add_argument('mntpoint')
    parser.add_argument('--noformat', dest='noformat', action='store_true')
    parser.add_argument('--useexisting', dest='useexisting',
                        action='store_true')
    parser.add_argument('--fstype', dest='fstype', action='store')
    parser.add_argument('--fsoptions', dest='fsoptions', action='store')
    parser.add_argument('--grow', dest='grow', action='store_true')
    parser.add_argument('--maxsize', dest='maxsize', action='store')
    parser.add_argument('--recommended', dest='recommended',
                        action='store_true')
    parser.add_argument('--percent', dest='percent', action='store_true')
    parser.add_argument('--encrypted', dest='encrypted', action='store_true')
    parser.add_argument('--passphrase', dest='passphrase', action='store')
    parser.add_argument('--escrowcert', dest='escrowcert', action='store')
    parser.add_argument('--backuppassphrase', dest='backuppassphrase',
                        action='store_true')
    parser.add_argument('--name', dest='name', action='store')
    parser.add_argument('--vgname', dest='vgname', action='store')
    parser.add_argument('--size', dest='size', action='store')
    parser.add_argument('--label', dest='label', action='store')

    args = clean_args(vars(parser.parse_args(rules)))
    parser = None
    return args


def parse_logging(rule):
    '''
    Parse the logging line
    '''
    parser = argparse.ArgumentParser()
    rules = shlex.split(rule)
    rules.pop(0)
    parser.add_argument('--host', dest='host', action='store')
    parser.add_argument('--port', dest='port', action='store')
    parser.add_argument('--level', dest='level', action='store',
                        choices=['debug', 'info', 'warning', 'error',
                                 'critical'])

    args = clean_args(vars(parser.parse_args(rules)))
    parser = None
    return args


def parse_monitor(rule):
    '''
    Parse the monitor line
    '''
    parser = argparse.ArgumentParser()
    rules = shlex.split(rule)
    rules.pop(0)
    parser.add_argument('--hsync', dest='hsync', action='store')
    parser.add_argument('--monitor', dest='monitor', action='store')
    parser.add_argument('--noprobe', dest='noprobe', action='store_true')
    parser.add_argument('--vsync', dest='vsync', action='store')

    args = clean_args(vars(parser.parse_args(rules)))
    parser = None
    return args


def parse_multipath(rule):
    '''
    Parse the multipath line
    '''
    parser = argparse.ArgumentParser()
    rules = shlex.split(rule)
    rules.pop(0)
    parser.add_argument('--name', dest='name', action='store')
    parser.add_argument('--device', dest='device', action='store')
    parser.add_argument('--rule', dest='rule', action='store')

    args = clean_args(vars(parser.parse_args(rules)))
    parser = None
    return args


def parse_network(rule):
    '''
    Parse the network line
    '''
    parser = argparse.ArgumentParser()
    rules = shlex.split(rule)
    rules.pop(0)
    parser.add_argument('--bootproto', dest='bootproto', action='store',
                        choices=['dhcp', 'bootp', 'static', 'ibft'])
    parser.add_argument('--device', dest='device', action='store')
    parser.add_argument('--ip', dest='ip', action='store')
    parser.add_argument('--ipv6', dest='ipv6', action='store')
    parser.add_argument('--gateway', dest='gateway', action='store')
    parser.add_argument('--nodefroute', dest='nodefroute', action='store_true')
    parser.add_argument('--nameserver', dest='nameserver', action='store')
    parser.add_argument('--nodns', dest='nodns', action='store_true')
    parser.add_argument('--netmask', dest='netmask', action='store')
    parser.add_argument('--hostname', dest='hostname', action='store')
    parser.add_argument('--ethtool', dest='ethtool', action='store')
    parser.add_argument('--essid', dest='essid', action='store')
    parser.add_argument('--wepkey', dest='wepkey', action='store')
    parser.add_argument('--wpakey', dest='wpakey', action='store')
    parser.add_argument('--onboot', dest='onboot', action='store')
    parser.add_argument('--dhcpclass', dest='dhcpclass', action='store')
    parser.add_argument('--mtu', dest='mtu', action='store')
    parser.add_argument('--noipv4', dest='noipv4', action='store_true')
    parser.add_argument('--noipv6', dest='noipv6', action='store_true')
    parser.add_argument('--activate', dest='activate', action='store_true')

    args = clean_args(vars(parser.parse_args(rules)))
    parser = None
    return args


def parse_nfs(rule):
    '''
    Parse the nfs line
    '''
    parser = argparse.ArgumentParser()
    rules = shlex.split(rule)
    rules.pop(0)
    parser.add_argument('--server', dest='server', action='store')
    parser.add_argument('--dir', dest='dir', action='store')
    parser.add_argument('--opts', dest='opts', action='store')

    args = clean_args(vars(parser.parse_args(rules)))
    parser = None
    return args


def parse_partition(rule):
    '''
    Parse the partition line
    '''
    parser = argparse.ArgumentParser()
    rules = shlex.split(rule)
    rules.pop(0)
    parser.add_argument('mntpoint')
    parser.add_argument('--size', dest='size', action='store')
    parser.add_argument('--grow', dest='grow', action='store_true')
    parser.add_argument('--maxsize', dest='maxsize', action='store')
    parser.add_argument('--noformat', dest='noformat', action='store_true')
    parser.add_argument('--onpart', '--usepart', dest='onpart', action='store')
    parser.add_argument('--ondisk', '--ondrive', dest='ondisk', action='store')
    parser.add_argument('--asprimary', dest='asprimary', action='store_true')
    parser.add_argument('--fsprofile', dest='fsprofile', action='store')
    parser.add_argument('--fstype', dest='fstype', action='store')
    parser.add_argument('--fsoptions', dest='fsoptions', action='store')
    parser.add_argument('--label', dest='label', action='store')
    parser.add_argument('--recommended', dest='recommended',
                        action='store_true')
    parser.add_argument('--onbiosdisk', dest='onbiosdisk', action='store')
    parser.add_argument('--encrypted', dest='encrypted', action='store_true')
    parser.add_argument('--passphrase', dest='passphrase', action='store')
    parser.add_argument('--escrowcert', dest='escrowcert', action='store')
    parser.add_argument('--backupphrase', dest='backupphrase', action='store')

    args = clean_args(vars(parser.parse_args(rules)))
    parser = None
    return args


def parse_raid(rule):
    '''
    Parse the raid line
    '''
    parser = argparse.ArgumentParser()
    rules = shlex.split(rule)
    rules.pop(0)

    partitions = []
    newrules = []
    for count in range(0, len(rules)):
        if count == 0:
            newrules.append(rules[count])
            continue
        elif rules[count].startswith('--'):
            newrules.append(rules[count])
            continue
        else:
            partitions.append(rules[count])
    rules = newrules

    parser.add_argument('mntpoint')
    parser.add_argument('--level', dest='level', action='store')
    parser.add_argument('--device', dest='device', action='store')
    parser.add_argument('--spares', dest='spares', action='store')
    parser.add_argument('--fstype', dest='fstype', action='store')
    parser.add_argument('--fsoptions', dest='fsoptions', action='store')
    parser.add_argument('--label', dest='label', action='store')
    parser.add_argument('--noformat', dest='noformat', action='store_true')
    parser.add_argument('--useexisting', dest='useexisting',
                        action='store_true')
    parser.add_argument('--encrypted', dest='encrypted', action='store_true')
    parser.add_argument('--passphrase', dest='passphrase', action='store')
    parser.add_argument('--escrowcert', dest='escrowcert', action='store')
    parser.add_argument('--backuppassphrase', dest='backuppassphrase',
                        action='store')

    args = clean_args(vars(parser.parse_args(rules)))
    if partitions:
        args['partitions'] = partitions
    parser = None
    return args


def parse_reboot(rule):
    '''
    Parse the reboot line
    '''
    parser = argparse.ArgumentParser()
    rules = shlex.split(rule)
    rules.pop(0)
    parser.add_argument('--eject', dest='eject', action='store_true')

    args = clean_args(vars(parser.parse_args(rules)))
    parser = None
    return args


def parse_repo(rule):
    '''
    Parse the repo line
    '''
    parser = argparse.ArgumentParser()
    rules = shlex.split(rule)
    rules.pop(0)
    parser.add_argument('--name', dest='name', action='store')
    parser.add_argument('--baseurl', dest='baseurl', action='store')
    parser.add_argument('--mirrorlist', dest='mirrorlist', action='store')
    parser.add_argument('--cost', dest='cost', action='store')
    parser.add_argument('--excludepkgs', dest='excludepkgs', action='store')
    parser.add_argument('--includepkgs', dest='includepkgs', action='store')
    parser.add_argument('--proxy', dest='proxy', action='store')
    parser.add_argument('--ignoregroups', dest='ignoregroups', action='store')
    parser.add_argument('--noverifyssl', dest='noverifyssl',
                        action='store_true')

    args = clean_args(vars(parser.parse_args(rules)))
    parser = None
    return args


def parse_rescue(rule):
    '''
    Parse the rescue line
    '''
    parser = argparse.ArgumentParser()
    rules = shlex.split(rule)
    rules.pop(0)
    parser.add_argument('--nomount', dest='nomount', action='store_true')
    parser.add_argument('--romount', dest='romount', action='store_true')

    args = clean_args(vars(parser.parse_args(rules)))
    parser = None
    return args


def parse_rootpw(rule):
    '''
    Parse the rootpw line
    '''
    parser = argparse.ArgumentParser()
    rules = shlex.split(rule)
    rules.pop(0)
    parser.add_argument('--iscrypted', dest='iscrypted', action='store_true')
    parser.add_argument('--plaintext', dest='plaintext', action='store_true')
    parser.add_argument('--lock', dest='lock', action='store_true')
    parser.add_argument('password')

    args = clean_args(vars(parser.parse_args(rules)))
    parser = None
    return args


def parse_selinux(rule):
    '''
    Parse the selinux line
    '''
    parser = argparse.ArgumentParser()
    rules = shlex.split(rule)
    rules.pop(0)
    parser.add_argument('--disabled', dest='disabled', action='store_true')
    parser.add_argument('--enforcing', dest='enforcing', action='store_true')
    parser.add_argument('--permissive', dest='permissive', action='store_true')

    args = clean_args(vars(parser.parse_args(rules)))
    parser = None
    return args


def parse_services(rule):
    '''
    Parse the services line
    '''
    parser = argparse.ArgumentParser()
    rules = shlex.split(rule)
    rules.pop(0)
    parser.add_argument('--disabled', dest='disabled', action='store')
    parser.add_argument('--enabled', dest='enabled', action='store')

    args = clean_args(vars(parser.parse_args(rules)))
    parser = None
    return args


def parse_sshpw(rule):
    '''
    Parse the sshpw line
    '''
    parser = argparse.ArgumentParser()
    rules = shlex.split(rule)
    rules.pop(0)
    parser.add_argument('--username', dest='username', action='store')
    parser.add_argument('--iscrypted', dest='iscrypted', action='store_true')
    parser.add_argument('--plaintext', dest='plaintext', action='store_true')
    parser.add_argument('--lock', dest='lock', action='store_true')

    args = clean_args(vars(parser.parse_args(rules)))
    parser = None
    return args


def parse_timezone(rule):
    '''
    Parse the timezone line
    '''
    parser = argparse.ArgumentParser()
    rules = shlex.split(rule)
    rules.pop(0)
    parser.add_argument('--utc', dest='utc', action='store_true')
    parser.add_argument('--nontp', dest='nontp', action='store_true')
    parser.add_argument('--ntpservers', dest='ntpservers', action='store')
    parser.add_argument('--isUtc', dest='isutc', action='store_true')
    parser.add_argument('timezone')

    args = clean_args(vars(parser.parse_args(rules)))
    parser = None
    return args


def parse_updates(rule):
    '''
    Parse the updates line
    '''
    rules = shlex.split(rule)
    rules.pop(0)
    if len(rules) > 0:
        return {'url': rules[0]}
    else:
        return True


def parse_upgrade(rule):
    '''
    Parse the upgrade line
    '''
    parser = argparse.ArgumentParser()
    rules = shlex.split(rule)
    rules.pop(0)
    parser.add_argument('--root-device', dest='root-device', action='store')

    args = clean_args(vars(parser.parse_args(rules)))
    parser = None
    if args:
        return args
    return True


def parse_url(rule):
    '''
    Parse the url line
    '''
    parser = argparse.ArgumentParser()
    rules = shlex.split(rule)
    rules.pop(0)
    parser.add_argument('--url', dest='url', action='store')
    parser.add_argument('--proxy', dest='proxy', action='store')
    parser.add_argument('--noverifyssl', dest='noverifyssl',
                        action='store_true')

    args = clean_args(vars(parser.parse_args(rules)))
    parser = None
    return args


def parse_user(rule):
    '''
    Parse the user line
    '''
    parser = argparse.ArgumentParser()
    rules = shlex.split(rule)
    rules.pop(0)
    parser.add_argument('--name', dest='name', action='store')
    parser.add_argument('--gecos', dest='gecos', action='store')
    parser.add_argument('--groups', dest='groups', action='store')
    parser.add_argument('--homedir', dest='homedir', action='store')
    parser.add_argument('--lock', dest='lock', action='store_true')
    parser.add_argument('--password', dest='password', action='store')
    parser.add_argument('--iscrypted', dest='iscrypted', action='store_true')
    parser.add_argument('--plaintext', dest='plaintext', action='store_true')
    parser.add_argument('--shell', dest='shell', action='store')
    parser.add_argument('--uid', dest='uid', action='store')

    args = clean_args(vars(parser.parse_args(rules)))
    parser = None
    return args


def parse_vnc(rule):
    '''
    Parse the vnc line
    '''
    parser = argparse.ArgumentParser()
    rules = shlex.split(rule)
    rules.pop(0)
    parser.add_argument('--host', dest='host', action='store')
    parser.add_argument('--port', dest='port', action='store')
    parser.add_argument('--password', dest='password', action='store')

    args = clean_args(vars(parser.parse_args(rules)))
    parser = None
    return args


def parse_volgroup(rule):
    '''
    Parse the volgroup line
    '''
    parser = argparse.ArgumentParser()
    rules = shlex.split(rule)
    rules.pop(0)

    partitions = []
    newrules = []
    for count in range(0, len(rules)):
        if count == 0:
            newrules.append(rules[count])
            continue
        elif rules[count].startswith('--'):
            newrules.append(rules[count])
            continue
        else:
            partitions.append(rules[count])
    rules = newrules

    parser.add_argument('name')
    parser.add_argument('--noformat', dest='noformat', action='store_true')
    parser.add_argument('--useexisting', dest='useexisting',
                        action='store_true')
    parser.add_argument('--pesize', dest='pesize', action='store')
    parser.add_argument('--reserved-space', dest='reserved-space',
                        action='store')
    parser.add_argument('--reserved-percent', dest='reserved-percent',
                        action='store')

    args = clean_args(vars(parser.parse_args(rules)))
    if partitions:
        args['partitions'] = partitions
    parser = None
    return args


def parse_xconfig(rule):
    '''
    Parse the xconfig line
    '''
    parser = argparse.ArgumentParser()
    rules = shlex.split(rule)
    rules.pop(0)
    parser.add_argument('--defaultdesktop', dest='defaultdesktop',
                        action='store')
    parser.add_argument('--startxonboot', dest='startxonboot',
                        action='store_true')

    args = clean_args(vars(parser.parse_args(rules)))
    parser = None
    return args


def parse_zfcp(rule):
    '''
    Parse the zfcp line
    '''
    parser = argparse.ArgumentParser()
    rules = shlex.split(rule)
    rules.pop(0)
    parser.add_argument('--devnum', dest='devnum', action='store')
    parser.add_argument('--fcplun', dest='fcplun', action='store')
    parser.add_argument('--wwpn', dest='wwpn', action='store')

    args = clean_args(vars(parser.parse_args(rules)))
    parser = None
    return args


def mksls(src, dst=None):
    '''
    Convert a kickstart file to an SLS file
    '''
    mode = 'command'
    sls = {}
    ks_opts = {}
    with salt.utils.fopen(src, 'r') as fh_:
        for line in fh_:
            if line.startswith('#'):
                continue

            if mode == 'command':
                if line.startswith('auth ') or line.startswith('authconfig '):
                    ks_opts['auth'] = parse_auth(line)
                elif line.startswith('autopart'):
                    ks_opts['autopath'] = parse_autopart(line)
                elif line.startswith('autostep'):
                    ks_opts['autostep'] = parse_autostep(line)
                elif line.startswith('bootloader'):
                    ks_opts['bootloader'] = parse_bootloader(line)
                elif line.startswith('btrfs'):
                    ks_opts['btrfs'] = parse_btrfs(line)
                elif line.startswith('cdrom'):
                    ks_opts['cdrom'] = True
                elif line.startswith('clearpart'):
                    ks_opts['clearpart'] = parse_clearpart(line)
                elif line.startswith('cmdline'):
                    ks_opts['cmdline'] = True
                elif line.startswith('device'):
                    ks_opts['device'] = parse_device(line)
                elif line.startswith('dmraid'):
                    ks_opts['dmraid'] = parse_dmraid(line)
                elif line.startswith('driverdisk'):
                    ks_opts['driverdisk'] = parse_driverdisk(line)
                elif line.startswith('firewall'):
                    ks_opts['firewall'] = parse_firewall(line)
                elif line.startswith('firstboot'):
                    ks_opts['firstboot'] = parse_firstboot(line)
                elif line.startswith('group'):
                    ks_opts['group'] = parse_group(line)
                elif line.startswith('graphical'):
                    ks_opts['graphical'] = True
                elif line.startswith('halt'):
                    ks_opts['halt'] = True
                elif line.startswith('harddrive'):
                    ks_opts['harddrive'] = True
                elif line.startswith('ignoredisk'):
                    ks_opts['ignoredisk'] = parse_ignoredisk(line)
                elif line.startswith('install'):
                    ks_opts['install'] = True
                elif line.startswith('iscsi'):
                    ks_opts['iscsi'] = parse_iscsi(line)
                elif line.startswith('iscsiname'):
                    ks_opts['iscsiname'] = parse_iscsiname(line)
                elif line.startswith('keyboard'):
                    ks_opts['keyboard'] = parse_keyboard(line)
                elif line.startswith('lang'):
                    ks_opts['lang'] = parse_lang(line)
                elif line.startswith('logvol'):
                    if 'logvol' not in ks_opts.keys():
                        ks_opts['logvol'] = []
                    ks_opts['logvol'].append(parse_logvol(line))
                elif line.startswith('logging'):
                    ks_opts['logging'] = parse_logging(line)
                elif line.startswith('mediacheck'):
                    ks_opts['mediacheck'] = True
                elif line.startswith('monitor'):
                    ks_opts['monitor'] = parse_monitor(line)
                elif line.startswith('multipath'):
                    ks_opts['multipath'] = parse_multipath(line)
                elif line.startswith('network'):
                    if 'network' not in ks_opts.keys():
                        ks_opts['network'] = []
                    ks_opts['network'].append(parse_network(line))
                elif line.startswith('nfs'):
                    ks_opts['nfs'] = True
                elif line.startswith('part ') or line.startswith('partition'):
                    if 'part' not in ks_opts.keys():
                        ks_opts['part'] = []
                    ks_opts['part'].append(parse_partition(line))
                elif line.startswith('poweroff'):
                    ks_opts['poweroff'] = True
                elif line.startswith('raid'):
                    if 'raid' not in ks_opts.keys():
                        ks_opts['raid'] = []
                    ks_opts['raid'].append(parse_raid(line))
                elif line.startswith('reboot'):
                    ks_opts['reboot'] = parse_reboot(line)
                elif line.startswith('repo'):
                    ks_opts['repo'] = parse_repo(line)
                elif line.startswith('rescue'):
                    ks_opts['rescue'] = parse_rescue(line)
                elif line.startswith('rootpw'):
                    ks_opts['rootpw'] = parse_rootpw(line)
                elif line.startswith('selinux'):
                    ks_opts['selinux'] = parse_selinux(line)
                elif line.startswith('services'):
                    ks_opts['services'] = parse_services(line)
                elif line.startswith('shutdown'):
                    ks_opts['shutdown'] = True
                elif line.startswith('sshpw'):
                    ks_opts['sshpw'] = parse_sshpw(line)
                elif line.startswith('skipx'):
                    ks_opts['skipx'] = True
                elif line.startswith('text'):
                    ks_opts['text'] = True
                elif line.startswith('timezone'):
                    ks_opts['timezone'] = parse_timezone(line)
                elif line.startswith('updates'):
                    ks_opts['updates'] = parse_updates(line)
                elif line.startswith('upgrade'):
                    ks_opts['upgrade'] = parse_upgrade(line)
                elif line.startswith('url'):
                    ks_opts['url'] = True
                elif line.startswith('user'):
                    ks_opts['user'] = parse_user(line)
                elif line.startswith('vnc'):
                    ks_opts['vnc'] = parse_vnc(line)
                elif line.startswith('volgroup'):
                    ks_opts['volgroup'] = parse_volgroup(line)
                elif line.startswith('xconfig'):
                    ks_opts['xconfig'] = parse_xconfig(line)
                elif line.startswith('zerombr'):
                    ks_opts['zerombr'] = True
                elif line.startswith('zfcp'):
                    ks_opts['zfcp'] = parse_zfcp(line)

            if line.startswith('%include'):
                rules = shlex.split(line)
                if not ks_opts['include']:
                    ks_opts['include'] = []
                ks_opts['include'].append(rules[1])

            if line.startswith('%ksappend'):
                rules = shlex.split(line)
                if not ks_opts['ksappend']:
                    ks_opts['ksappend'] = []
                ks_opts['ksappend'].append(rules[1])

            if line.startswith('%packages'):
                mode = 'packages'
                if 'packages' not in ks_opts.keys():
                    ks_opts['packages'] = {'packages': {}}

                parser = argparse.ArgumentParser()
                opts = shlex.split(line)
                opts.pop(0)
                parser.add_argument('--default', dest='default', action='store_true')
                parser.add_argument('--excludedocs', dest='excludedocs',
                                    action='store_true')
                parser.add_argument('--ignoremissing', dest='ignoremissing',
                                    action='store_true')
                parser.add_argument('--instLangs', dest='instLangs', action='store')
                parser.add_argument('--multilib', dest='multilib', action='store_true')
                parser.add_argument('--nodefaults', dest='nodefaults',
                                    action='store_true')
                parser.add_argument('--optional', dest='optional', action='store_true')
                parser.add_argument('--nobase', dest='nobase', action='store_true')
                args = clean_args(vars(parser.parse_args(opts)))
                ks_opts['packages']['options'] = args

                continue

            if line.startswith('%pre'):
                mode = 'pre'

                parser = argparse.ArgumentParser()
                opts = shlex.split(line)
                opts.pop(0)
                parser.add_argument('--interpreter', dest='interpreter',
                                    action='store')
                parser.add_argument('--erroronfail', dest='erroronfail',
                                    action='store_true')
                parser.add_argument('--log', dest='log', action='store')
                args = clean_args(vars(parser.parse_args(opts)))
                ks_opts['pre'] = {'options': args, 'script': ''}

                continue

            if line.startswith('%post'):
                mode = 'post'

                parser = argparse.ArgumentParser()
                opts = shlex.split(line)
                opts.pop(0)
                parser.add_argument('--nochroot', dest='nochroot', action='store_true')
                parser.add_argument('--interpreter', dest='interpreter',
                                    action='store')
                parser.add_argument('--erroronfail', dest='erroronfail',
                                    action='store_true')
                parser.add_argument('--log', dest='log', action='store')
                args = clean_args(vars(parser.parse_args(opts)))
                ks_opts['post'] = {'options': args, 'script': ''}

                continue

            if line.startswith('%end'):
                mode = None

            if mode == 'packages':
                if line.startswith('-'):
                    package = line.replace('-', '', 1).strip()
                    ks_opts['packages']['packages'][package] = False
                else:
                    ks_opts['packages']['packages'][line.strip()] = True

            if mode == 'pre':
                ks_opts['pre']['script'] += line

            if mode == 'post':
                ks_opts['post']['script'] += line

    # Set language
    sls[ks_opts['lang']['lang']] = {'locale': ['system']}

    # Set keyboard
    sls[ks_opts['keyboard']['xlayouts']] = {'keyboard': ['system']}

    # Set timezone
    sls[ks_opts['timezone']['timezone']] = {'timezone': ['system']}
    if 'utc' in ks_opts['timezone'].keys():
        sls[ks_opts['timezone']['timezone']]['timezone'].append('utc')

    # Set network
    if 'network' in ks_opts.keys():
        for interface in ks_opts['network']:
            device = interface.get('device', None)
            if device is not None:
                del interface['device']
                sls[device] = {'proto': interface['bootproto']}
                del interface['bootproto']

                if 'onboot' in interface.keys():
                    if 'no' in interface['onboot']:
                        sls[device]['enabled'] = False
                    else:
                        sls[device]['enabled'] = True
                    del interface['onboot']

                if 'noipv4' in interface.keys():
                    sls[device]['ipv4'] = {'enabled': False}
                    del interface['noipv4']
                if 'noipv6' in interface.keys():
                    sls[device]['ipv6'] = {'enabled': False}
                    del interface['noipv6']

                for option in interface.keys():
                    if type(interface[option]) is bool:
                        sls[device][option] = {'enabled': [interface[option]]}
                    else:
                        sls[device][option] = interface[option]
            if 'hostname' in interface:
                sls['system'] = {
                    'network.system': {
                        'enabled': True,
                        'hostname': interface['hostname'],
                        'apply_hostname': True,
                    }
                }

    # Set selinux
    if 'selinux' in ks_opts.keys():
        for mode in ks_opts['selinux']:
            sls[mode] = {'selinux': ['mode']}

    # Get package data together
    if 'nobase' not in ks_opts['packages']['options']:
        sls['base'] = {'pkg_group': ['installed']}

    packages = ks_opts['packages']['packages']
    for package in packages:
        if not packages[package]:
            continue
        if package and packages[package] is True:
            if package.startswith('@'):
                pkg_group = package.replace('@', '', 1)
                sls[pkg_group] = {'pkg_group': ['installed']}
            else:
                sls[package] = {'pkg': ['installed']}
        elif packages[package] is False:
            sls[package] = {'pkg': ['absent']}

    if dst:
        with salt.utils.fopen(dst, 'w') as fp_:
            fp_.write(yaml.safe_dump(sls, default_flow_style=False))
    else:
        return yaml.safe_dump(sls, default_flow_style=False)



# -*- coding: utf-8 -*-
'''
:codeauthor: :email:`Alexandru Bleotu <alexandru.bleotu@morganstanley.com>`

Tests for cluster related functions in salt.utils.vmware
'''

# Import python libraries
from __future__ import absolute_import
import logging
# Import Salt testing libraries
from salttesting import TestCase, skipIf
from salttesting.mock import NO_MOCK, NO_MOCK_REASON, patch, MagicMock, call
# Import Salt libraries
from salt.exceptions import VMwareApiError, VMwareRuntimeError, \
        VMwareObjectRetrievalError
import salt.utils.vmware as vmware
# Import Third Party Libs
try:
    from pyVmomi import vim, vmodl
    HAS_PYVMOMI = True
except ImportError:
    HAS_PYVMOMI = False

# Get Logging Started
log = logging.getLogger(__name__)


@skipIf(NO_MOCK, NO_MOCK_REASON)
@skipIf(not HAS_PYVMOMI, 'The \'pyvmomi\' library is missing')
@patch('salt.utils.vmware.get_managed_object_name',
       MagicMock())
@patch('salt.utils.vmware.get_service_instance_from_managed_object',
       MagicMock())
@patch('salt.utils.vmware.get_mors_with_properties',
       MagicMock(return_value=[{'name': 'fake_cluster',
                                'object': MagicMock()}]))
class GetClusterTestCase(TestCase):
    '''Tests for salt.utils.vmware.get_cluster'''

    def setUp(self):
        self.mock_si = MagicMock()
        self.mock_dc = MagicMock()
        self.mock_cluster1 = MagicMock()
        self.mock_cluster2 = MagicMock()
        self.mock_entries = [{'name': 'fake_cluster1',
                              'object': self.mock_cluster1},
                             {'name': 'fake_cluster2',
                              'object': self.mock_cluster2}]

    def test_get_managed_object_name_call(self):
        mock_get_managed_object_name = MagicMock()
        with patch('salt.utils.vmware.get_managed_object_name',
                   mock_get_managed_object_name):
            vmware.get_cluster(self.mock_dc, 'fake_cluster')
        mock_get_managed_object_name.assert_called_once_with(self.mock_dc)

    def test_get_service_instance_from_managed_object(self):
        mock_dc_name = MagicMock()
        mock_get_service_instance_from_managed_object = MagicMock()
        with patch('salt.utils.vmware.get_managed_object_name',
                   MagicMock(return_value=mock_dc_name)):
            with patch(
                'salt.utils.vmware.get_service_instance_from_managed_object',
                mock_get_service_instance_from_managed_object):

                vmware.get_cluster(self.mock_dc, 'fake_cluster')
        mock_get_service_instance_from_managed_object.assert_called_once_with(
            self.mock_dc, name=mock_dc_name)

    def test_traversal_spec_init(self):
        mock_dc_name = MagicMock()
        mock_traversal_spec = MagicMock()
        mock_traversal_spec_ini = MagicMock(return_value=mock_traversal_spec)
        mock_get_service_instance_from_managed_object = MagicMock()
        patch_traversal_spec_str = \
                'salt.utils.vmware.vmodl.query.PropertyCollector.TraversalSpec'

        with patch(patch_traversal_spec_str, mock_traversal_spec_ini):
            vmware.get_cluster(self.mock_dc, 'fake_cluster')
        mock_traversal_spec_ini.assert_has_calls(
            [call(path='childEntity',
                  skip=False,
                  type=vim.Folder),
            call(path='hostFolder',
                  skip=True,
                  type=vim.Datacenter,
                  selectSet=[mock_traversal_spec])])

    def test_get_mors_with_properties_call(self):
        mock_get_mors_with_properties = MagicMock(
            return_value=[{'name': 'fake_cluster', 'object': MagicMock()}])
        mock_traversal_spec = MagicMock()
        patch_traversal_spec_str = \
                'salt.utils.vmware.vmodl.query.PropertyCollector.TraversalSpec'
        with patch(
            'salt.utils.vmware.get_service_instance_from_managed_object',
            MagicMock(return_value=self.mock_si)):

            with patch('salt.utils.vmware.get_mors_with_properties',
                       mock_get_mors_with_properties):
                with patch(patch_traversal_spec_str,
                           MagicMock(return_value=mock_traversal_spec)):

                    vmware.get_cluster(self.mock_dc, 'fake_cluster')
        mock_get_mors_with_properties.assert_called_once_with(
            self.mock_si, vim.ClusterComputeResource,
            container_ref=self.mock_dc,
            property_list=['name'],
            traversal_spec=mock_traversal_spec)

    def test_get_mors_with_properties_returns_empty_array(self):
        with patch('salt.utils.vmware.get_managed_object_name',
                   MagicMock(return_value='fake_dc')):
            with patch('salt.utils.vmware.get_mors_with_properties',
                       MagicMock(return_value=[])):
                with self.assertRaises(VMwareObjectRetrievalError) as excinfo:
                    vmware.get_cluster(self.mock_dc, 'fake_cluster')
        self.assertEqual(excinfo.exception.strerror,
                         'Cluster \'fake_cluster\' was not found in '
                         'datacenter \'fake_dc\'')

    def test_cluster_not_found(self):
        with patch('salt.utils.vmware.get_managed_object_name',
                   MagicMock(return_value='fake_dc')):
            with patch('salt.utils.vmware.get_mors_with_properties',
                       MagicMock(return_value=self.mock_entries)):
                with self.assertRaises(VMwareObjectRetrievalError) as excinfo:
                    vmware.get_cluster(self.mock_dc, 'fake_cluster')
        self.assertEqual(excinfo.exception.strerror,
                         'Cluster \'fake_cluster\' was not found in '
                         'datacenter \'fake_dc\'')

    def test_cluster_found(self):
        with patch('salt.utils.vmware.get_managed_object_name',
                   MagicMock(return_value='fake_dc')):
            with patch('salt.utils.vmware.get_mors_with_properties',
                       MagicMock(return_value=self.mock_entries)):
                res = vmware.get_cluster(self.mock_dc, 'fake_cluster2')
        self.assertEqual(res, self.mock_cluster2)


@skipIf(not HAS_PYVMOMI, 'The \'pyvmomi\' library is missing')
@patch('salt.utils.vmware.get_managed_object_name', MagicMock())
class CreateClusterTestCase(TestCase):
    '''Tests for salt.utils.vmware.create_cluster'''

    def setUp(self):
        self.mock_create_cluster_ex = MagicMock()
        self.mock_dc = MagicMock(
            hostFolder=MagicMock(CreateClusterEx=self.mock_create_cluster_ex))
        self.mock_cluster_spec = MagicMock()

    def test_get_managed_object_name(self):
        mock_get_managed_object_name = MagicMock()
        with patch('salt.utils.vmware.get_managed_object_name',
                   mock_get_managed_object_name):
            vmware.create_cluster(self.mock_dc, 'fake_cluster',
                                  self.mock_cluster_spec)
        mock_get_managed_object_name.assert_called_once_with(self.mock_dc)

    def test_create_cluster_call(self):
        vmware.create_cluster(self.mock_dc, 'fake_cluster',
                              self.mock_cluster_spec)
        self.mock_create_cluster_ex.assert_called_once_with(
           'fake_cluster', self.mock_cluster_spec)

    def test_create_cluster_raise_vim_fault(self):
        exc = vim.fault.VimFault()
        exc.msg = 'VimFault msg'
        self.mock_dc.hostFolder.CreateClusterEx = MagicMock(
            side_effect=exc)
        with self.assertRaises(VMwareApiError) as excinfo:
            vmware.create_cluster(self.mock_dc, 'fake_cluster',
                                  self.mock_cluster_spec)
        self.assertEqual(excinfo.exception.strerror, 'VimFault msg')

    def test_create_cluster_raise_runtime_fault(self):
        exc = vmodl.RuntimeFault()
        exc.msg = 'RuntimeFault msg'
        self.mock_dc.hostFolder.CreateClusterEx = MagicMock(
            side_effect=exc)
        with self.assertRaises(VMwareRuntimeError) as excinfo:
            vmware.create_cluster(self.mock_dc, 'fake_cluster',
                                  self.mock_cluster_spec)
        self.assertEqual(excinfo.exception.strerror, 'RuntimeFault msg')


@skipIf(NO_MOCK, NO_MOCK_REASON)
@skipIf(not HAS_PYVMOMI, 'The \'pyvmomi\' library is missing')
@patch('salt.utils.vmware.get_managed_object_name', MagicMock())
@patch('salt.utils.vmware.wait_for_task', MagicMock())
class UpdateClusterTestCase(TestCase):
    '''Tests for salt.utils.vmware.update_cluster'''

    def setUp(self):
        self.mock_task = MagicMock()
        self.mock_reconfigure_compute_resource_task = \
                MagicMock(return_value=self.mock_task)
        self.mock_cluster = MagicMock(ReconfigureComputeResource_Task=
            self.mock_reconfigure_compute_resource_task)
        self.mock_cluster_spec = MagicMock()

    def test_get_managed_object_name(self):
        mock_get_managed_object_name = MagicMock()
        with patch('salt.utils.vmware.get_managed_object_name',
                   mock_get_managed_object_name):
            vmware.update_cluster(self.mock_cluster, self.mock_cluster_spec)
        mock_get_managed_object_name.assert_called_once_with(self.mock_cluster)

    def test_reconfigure_compute_resource_task_call(self):
        vmware.update_cluster(self.mock_cluster, self.mock_cluster_spec)
        self.mock_reconfigure_compute_resource_task.assert_called_once_with(
            self.mock_cluster_spec, modify=True)

    def test_reconfigure_compute_resource_raise_vim_fault(self):
        exc = vim.fault.VimFault()
        exc.msg = 'VimFault msg'
        self.mock_cluster.ReconfigureComputeResource_Task = \
                MagicMock(side_effect=exc)
        with self.assertRaises(VMwareApiError) as excinfo:
            vmware.update_cluster(self.mock_cluster, self.mock_cluster_spec)
        self.assertEqual(excinfo.exception.strerror, 'VimFault msg')

    def test_reconfigure_compute_resource_raise_runtime_fault(self):
        exc = vmodl.RuntimeFault()
        exc.msg = 'RuntimeFault msg'
        self.mock_cluster.ReconfigureComputeResource_Task = \
                MagicMock(side_effect=exc)
        with self.assertRaises(VMwareRuntimeError) as excinfo:
            vmware.update_cluster(self.mock_cluster, self.mock_cluster_spec)
        self.assertEqual(excinfo.exception.strerror, 'RuntimeFault msg')

    def test_wait_for_task_call(self):
        mock_wait_for_task = MagicMock()
        with patch('salt.utils.vmware.get_managed_object_name',
                   MagicMock(return_value='fake_cluster')):
            with patch('salt.utils.vmware.wait_for_task', mock_wait_for_task):
                vmware.update_cluster(self.mock_cluster,
                                      self.mock_cluster_spec)
        mock_wait_for_task.assert_called_once_with(
            self.mock_task, 'fake_cluster', 'ClusterUpdateTask')


if __name__ == '__main__':
    from integration import run_tests
    run_tests(GetClusterTestCase, needs_daemon=False)
    run_tests(CreateClusterTestCase, needs_daemon=False)
    run_tests(UpdateClusterTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import

# Import Salt Libs
from salt.states import mac_xattr as xattr

# Import Salt Testing Libs
from salttesting import TestCase
from salttesting.helpers import ensure_in_syspath
from salttesting.mock import (
    MagicMock,
    patch
)

ensure_in_syspath('../../')

xattr.__salt__ = {}


class XAttrTestCase(TestCase):

    @patch('os.path.exists')
    def test_exists_not(self, exists_mock):
        '''
            Test adding an attribute when it doesn't exist
        '''
        expected = {
            'changes': {'key': 'value'},
            'comment': '',
            'name': '/path/to/file',
            'result': True
        }

        exists_mock.return_value = True

        list_mock = MagicMock(return_value={'other.id': 'value2'})
        write_mock = MagicMock()
        with patch.dict(xattr.__salt__, {'xattr.list': list_mock,
                                         'xattr.write': write_mock}):
            out = xattr.exists('/path/to/file', ['key=value'])

            list_mock.assert_called_once_with('/path/to/file')
            write_mock.assert_called_once_with('/path/to/file', 'key', 'value', False)
            self.assertEqual(out, expected)

    @patch('os.path.exists')
    def test_exists_change(self, exists_mock):
        '''
            Test changing and attribute value
        '''
        expected = {
            'changes': {'key': 'other_value'},
            'comment': '',
            'name': '/path/to/file',
            'result': True
        }

        exists_mock.return_value = True

        list_mock = MagicMock(return_value={'key': 'value'})
        write_mock = MagicMock()
        with patch.dict(xattr.__salt__, {'xattr.list': list_mock,
                                         'xattr.write': write_mock}):
            out = xattr.exists('/path/to/file', ['key=other_value'])

            list_mock.assert_called_once_with('/path/to/file')
            write_mock.assert_called_once_with('/path/to/file', 'key', 'other_value', False)
            self.assertEqual(out, expected)

    @patch('os.path.exists')
    def test_exists_already(self, exists_mock):
        '''
            Test that having the same value does nothing
        '''
        expected = {
            'changes': {},
            'comment': 'All values existed correctly.',
            'name': '/path/to/file',
            'result': True
        }

        exists_mock.return_value = True

        list_mock = MagicMock(return_value={'key': 'value'})
        write_mock = MagicMock()
        with patch.dict(xattr.__salt__, {'xattr.list': list_mock,
                                         'xattr.write': write_mock}):
            out = xattr.exists('/path/to/file', ['key=value'])

            list_mock.assert_called_once_with('/path/to/file')
            assert not write_mock.called
            self.assertEqual(out, expected)

    @patch('os.path.exists')
    def test_delete(self, exists_mock):
        '''
            Test deleting an attribute from a file
        '''
        expected = {
            'changes': {'key': 'delete'},
            'comment': '',
            'name': '/path/to/file',
            'result': True
        }

        exists_mock.return_value = True

        list_mock = MagicMock(return_value={'key': 'value2'})
        delete_mock = MagicMock()
        with patch.dict(xattr.__salt__, {'xattr.list': list_mock,
                                         'xattr.delete': delete_mock}):
            out = xattr.delete('/path/to/file', ['key'])

            list_mock.assert_called_once_with('/path/to/file')
            delete_mock.assert_called_once_with('/path/to/file', 'key')
            self.assertEqual(out, expected)

    @patch('os.path.exists')
    def test_delete_not(self, exists_mock):
        '''
            Test deleting an attribute that doesn't exist from a file
        '''
        expected = {
            'changes': {},
            'comment': 'All attributes were already deleted.',
            'name': '/path/to/file',
            'result': True
        }

        exists_mock.return_value = True

        list_mock = MagicMock(return_value={'other.key': 'value2'})
        delete_mock = MagicMock()
        with patch.dict(xattr.__salt__, {'xattr.list': list_mock,
                                         'xattr.delete': delete_mock}):
            out = xattr.delete('/path/to/file', ['key'])

            list_mock.assert_called_once_with('/path/to/file')
            assert not delete_mock.called
            self.assertEqual(out, expected)


if __name__ == '__main__':
    from integration import run_tests
    run_tests(XAttrTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-

# Import python libs
from __future__ import absolute_import

# Import Salt Testing libs
from salttesting.helpers import ensure_in_syspath
ensure_in_syspath('../../')

# Import salt libs
import integration


class SysModuleTest(integration.ModuleCase):
    '''
    Validate the sys module
    '''
    def test_valid_docs(self):
        '''
        Make sure no functions are exposed that don't have valid docstrings
        '''
        ret = self.run_function('runtests_helpers.get_invalid_docs')
        if ret == {'missing_docstring': [], 'missing_cli_example': []}:
            return

        raise AssertionError(
            'There are some functions which do not have a docstring or do not '
            'have an example:\nNo docstring:\n{0}\nNo example:\n{1}\n'.format(
                '\n'.join(['  - {0}'.format(f) for f in ret['missing_docstring']]),
                '\n'.join(['  - {0}'.format(f) for f in ret['missing_cli_example']]),
            )
        )


if __name__ == '__main__':
    from integration import run_tests
    run_tests(SysModuleTest)



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Jayesh Kariya <jayeshk@saltstack.com>`
'''
# Import Python libs
from __future__ import absolute_import
import os

# Import Salt Testing Libs
from salttesting import skipIf, TestCase
from salttesting.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    mock_open,
    patch)

from salttesting.helpers import ensure_in_syspath

ensure_in_syspath('../../')

# Import Salt Libs
from salt.states import virt
import salt.utils

virt.__salt__ = {}
virt.__opts__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class LibvirtTestCase(TestCase):
    '''
    Test cases for salt.states.libvirt
    '''
    # 'keys' function tests: 1

    @skipIf(os.geteuid() != 0, 'you must be root to run this test')
    @patch('os.path.isfile', MagicMock(return_value=False))
    def test_keys(self):
        '''
        Test to manage libvirt keys.
        '''
        name = 'sunrise'

        ret = {'name': name,
               'result': True,
               'comment': '',
               'changes': {}}

        mock = MagicMock(side_effect=[[], ['libvirt.servercert.pem'],
                                      {'libvirt.servercert.pem': 'A'}])
        with patch.dict(virt.__salt__, {'pillar.ext': mock}):
            comt = ('All keys are correct')
            ret.update({'comment': comt})
            self.assertDictEqual(virt.keys(name), ret)

            with patch.dict(virt.__opts__, {'test': True}):
                comt = ('Libvirt keys are set to be updated')
                ret.update({'comment': comt, 'result': None})
                self.assertDictEqual(virt.keys(name), ret)

            with patch.dict(virt.__opts__, {'test': False}):
                with patch.object(salt.utils, 'fopen', MagicMock(mock_open())):
                    comt = ('Updated libvirt certs and keys')
                    ret.update({'comment': comt, 'result': True,
                                'changes': {'servercert': 'new'}})
                    self.assertDictEqual(virt.keys(name), ret)


if __name__ == '__main__':
    from integration import run_tests
    run_tests(LibvirtTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
Module for managing SNMP service settings on Windows servers.

'''

# Import python libs
from __future__ import absolute_import


def __virtual__():
    '''
    Load only on minions that have the win_snmp module.
    '''
    if 'win_snmp.get_agent_settings' in __salt__:
        return True
    return False


def agent_settings(name, contact, location, services=None):
    '''
    Manage the SNMP sysContact, sysLocation, and sysServices settings.

    :param str contact: The SNMP contact.
    :param str location: The SNMP location.
    :param str services: A list of selected services.

    Example of usage:

    .. code-block:: yaml

        snmp-agent-settings:
            win_snmp.agent_settings:
                - contact: Test Contact
                - location: Test Location
                - services:
                    - Physical
                    - Internet
    '''
    ret = {'name': name,
           'changes': {},
           'comment': str(),
           'result': None}

    ret_settings = {'changes': dict(),
                    'failures': dict()}

    if not services:
        services = ['None']

    # Filter services for unique items, and sort them for comparison purposes.
    services = sorted(set(services))

    settings = {'contact': contact, 'location': location, 'services': services}

    current_settings = __salt__['win_snmp.get_agent_settings']()

    for setting in settings:
        if str(settings[setting]) != str(current_settings[setting]):
            ret_settings['changes'][setting] = {'old': current_settings[setting],
                                                'new': settings[setting]}
    if not ret_settings['changes']:
        ret['comment'] = 'Agent settings already contain the provided values.'
        ret['result'] = True
        return ret
    elif __opts__['test']:
        ret['comment'] = 'Agent settings will be changed.'
        ret['changes'] = ret_settings
        return ret

    __salt__['win_snmp.set_agent_settings'](**settings)
    new_settings = __salt__['win_snmp.get_agent_settings']()

    for setting in settings:
        if settings[setting] != new_settings[setting]:
            ret_settings['failures'][setting] = {'old': current_settings[setting],
                                                 'new': new_settings[setting]}
            ret_settings['changes'].pop(setting, None)

    if ret_settings['failures']:
        ret['comment'] = 'Some agent settings failed to change.'
        ret['changes'] = ret_settings
        ret['result'] = False
    else:
        ret['comment'] = 'Set agent settings to contain the provided values.'
        ret['changes'] = ret_settings['changes']
        ret['result'] = True
    return ret


def auth_traps_enabled(name, status=True):
    '''
    Manage the sending of authentication traps.

    :param bool status: The enabled status.

    Example of usage:

    .. code-block:: yaml

        snmp-auth-traps:
            win_snmp.auth_traps_enabled:
                - status: True
    '''
    ret = {'name': name,
           'changes': {},
           'comment': str(),
           'result': None}

    vname = 'EnableAuthenticationTraps'
    current_status = __salt__['win_snmp.get_auth_traps_enabled']()

    if status == current_status:
        ret['comment'] = '{0} already contains the provided value.'.format(vname)
        ret['result'] = True
    elif __opts__['test']:
        ret['comment'] = '{0} will be changed.'.format(vname)
        ret['changes'] = {'old': current_status,
                          'new': status}
    else:
        ret['comment'] = 'Set {0} to contain the provided value.'.format(vname)
        ret['changes'] = {'old': current_status,
                          'new': status}
        ret['result'] = __salt__['win_snmp.set_auth_traps_enabled'](status=status)

    return ret


def community_names(name, communities=None):
    '''
    Manage the SNMP accepted community names and their permissions.

    :param str communities: A dictionary of SNMP communities and permissions.

    Example of usage:

    .. code-block:: yaml

        snmp-community-names:
            win_snmp.community_names:
                - communities:
                    TestCommunity: Read Only
                    OtherCommunity: Read Write
    '''
    ret = {'name': name,
           'changes': dict(),
           'comment': str(),
           'result': None}

    ret_communities = {'changes': dict(),
                       'failures': dict()}

    if not communities:
        communities = dict()

    current_communities = __salt__['win_snmp.get_community_names']()

    # Note any existing communities that should be removed.
    for current_vname in current_communities:
        if current_vname not in communities:
            ret_communities['changes'][current_vname] = {'old': current_communities[current_vname],
                                                         'new': None}

    # Note any new communities or existing communities that should be changed.
    for vname in communities:
        current_vdata = None
        if vname in current_communities:
            current_vdata = current_communities[vname]
        if communities[vname] != current_vdata:
            ret_communities['changes'][vname] = {'old': current_vdata,
                                                 'new': communities[vname]}

    if not ret_communities['changes']:
        ret['comment'] = 'Communities already contain the provided values.'
        ret['result'] = True
        return ret
    elif __opts__['test']:
        ret['comment'] = 'Communities will be changed.'
        ret['changes'] = ret_communities
        return ret

    __salt__['win_snmp.set_community_names'](communities=communities)
    new_communities = __salt__['win_snmp.get_community_names']()

    # Verify that any communities that needed to be removed were removed.
    for new_vname in new_communities:
        if new_vname not in communities:
            ret_communities['failures'][new_vname] = {'old': current_communities[new_vname],
                                                      'new': new_communities[new_vname]}
            ret_communities['changes'].pop(new_vname, None)

    # Verify that any new communities or existing communities that
    # needed to be changed were changed.
    for vname in communities:
        new_vdata = None
        if vname in new_communities:
            new_vdata = new_communities[vname]
        if communities[vname] != new_vdata:
            ret_communities['failures'][vname] = {'old': current_communities[vname],
                                                  'new': new_vdata}
            ret_communities['changes'].pop(vname, None)

    if ret_communities['failures']:
        ret['comment'] = 'Some communities failed to change.'
        ret['changes'] = ret_communities
        ret['result'] = False
    else:
        ret['comment'] = 'Set communities to contain the provided values.'
        ret['changes'] = ret_communities['changes']
        ret['result'] = True
    return ret



# -*- coding: utf-8 -*-
'''
Manage IAM roles
================

.. versionadded:: 2014.7.0

This module uses ``boto``, which can be installed via package, or pip.

This module accepts explicit IAM credentials but can also utilize
IAM roles assigned to the instance through Instance Profiles. Dynamic
credentials are then automatically obtained from AWS API and no further
configuration is necessary. More information available `here
<http://docs.aws.amazon.com/AWSEC2/latest/UserGuide/iam-roles-for-amazon-ec2.html>`_.

If IAM roles are not used you need to specify them either in a pillar file or
in the minion's config file:

.. code-block:: yaml

    iam.keyid: GKTADJGHEIQSXMKKRBJ08H
    iam.key: askdjghsdfjkghWupUjasdflkdfklgjsdfjajkghs

It's also possible to specify ``key``, ``keyid`` and ``region`` via a profile, either
passed in as a dict, or as a string to pull from pillars or minion config:

.. code-block:: yaml

    myprofile:
        keyid: GKTADJGHEIQSXMKKRBJ08H
        key: askjghsdfjkghWupUjasdflkdfklgjsdfjajkghs
        region: us-east-1

Creating a role will automatically create an instance profile and associate it
with the role. This is the default behavior of the AWS console.

.. code-block:: yaml

    myrole:
        boto_iam_role.present:
            - region: us-east-1
            - key: GKTADJGHEIQSXMKKRBJ08H
            - keyid: askdjghsdfjkghWupUjasdflkdfklgjsdfjajkghs
            - policies_from_pillars:
                - shared_iam_bootstrap_policy
            - policies:
                MySQSPolicy:
                    Statement:
                      - Action:
                            - sqs:*
                        Effect: Allow
                        Resource:
                            - arn:aws:sqs:*:*:*
                        Sid: MyPolicySQS1
                MyS3Policy:
                    Statement:
                      - Action:
                            - s3:GetObject
                        Effect: Allow
                        Resource:
                            - arn:aws:s3:*:*:mybucket/*

    # Using a credentials profile from pillars
    myrole:
        boto_iam_role.present:
            - profile: myiamprofile

    # Passing in a credentials profile
    myrole:
        boto_iam_role.present:
            - profile:
                key: GKTADJGHEIQSXMKKRBJ08H
                keyid: askdjghsdfjkghWupUjasdflkdfklgjsdfjajkghs
                region: us-east-1

If ``delete_policies: False`` is specified, existing policies that are not in
the given list of policies will not be deleted. This allows manual modifications
on the IAM role to be persistent. This functionality was added in 2015.8.0.

.. note::

    When using the ``profile`` parameter and ``region`` is set outside of
    the profile group, region is ignored and a default region will be used.

    If ``region`` is missing from the ``profile`` data set, ``us-east-1``
    will be used as the default region.

'''
from __future__ import absolute_import
import logging
import salt.utils.dictupdate as dictupdate
import salt.ext.six as six

log = logging.getLogger(__name__)


def __virtual__():
    '''
    Only load if boto is available.
    '''
    return 'boto_iam_role' if 'boto_iam.role_exists' in __salt__ else False


def present(
        name,
        policy_document=None,
        path=None,
        policies=None,
        policies_from_pillars=None,
        managed_policies=None,
        create_instance_profile=True,
        region=None,
        key=None,
        keyid=None,
        profile=None,
        delete_policies=True):
    '''
    Ensure the IAM role exists.

    name
        Name of the IAM role.

    policy_document
        The policy that grants an entity permission to assume the role. (See https://boto.readthedocs.io/en/latest/ref/iam.html#boto.iam.connection.IAMConnection.create_role)

    path
        The path to the role/instance profile. (See https://boto.readthedocs.io/en/latest/ref/iam.html#boto.iam.connection.IAMConnection.create_role)

    policies
        A dict of IAM role policies.

    policies_from_pillars
        A list of pillars that contain role policy dicts. Policies in the
        pillars will be merged in the order defined in the list and key
        conflicts will be handled by later defined keys overriding earlier
        defined keys. The policies defined here will be merged with the
        policies defined in the policies argument. If keys conflict, the keys
        in the policies argument will override the keys defined in
        policies_from_pillars.

    managed_policies
        A list of (AWS or Customer) managed policies to be attached to the role.

    create_instance_profile
        A boolean of whether or not to create an instance profile and associate
        it with this role.

    region
        Region to connect to.

    key
        Secret key to be used.

    keyid
        Access key to be used.

    profile
        A dict with region, key and keyid, or a pillar key (string)
        that contains a dict with region, key and keyid.

    delete_policies
        Deletes existing policies that are not in the given list of policies. Default
        value is ``True``. If ``False`` is specified, existing policies will not be deleted
        allowing manual modifications on the IAM role to be persistent.

        .. versionadded:: 2015.8.0
    '''
    ret = {'name': name, 'result': True, 'comment': '', 'changes': {}}
    _ret = _role_present(name, policy_document, path, region, key, keyid,
                         profile)
    if not policies:
        policies = {}
    if not policies_from_pillars:
        policies_from_pillars = []
    if not managed_policies:
        managed_policies = []
    _policies = {}
    for policy in policies_from_pillars:
        _policy = __salt__['pillar.get'](policy)
        _policies.update(_policy)
    _policies.update(policies)
    ret['changes'] = _ret['changes']
    ret['comment'] = ' '.join([ret['comment'], _ret['comment']])
    if not _ret['result']:
        ret['result'] = _ret['result']
        if ret['result'] is False:
            return ret
    if create_instance_profile:
        _ret = _instance_profile_present(name, region, key, keyid, profile)
        ret['changes'] = dictupdate.update(ret['changes'], _ret['changes'])
        ret['comment'] = ' '.join([ret['comment'], _ret['comment']])
        if not _ret['result']:
            ret['result'] = _ret['result']
            if ret['result'] is False:
                return ret
        _ret = _instance_profile_associated(name, region, key, keyid, profile)
        ret['changes'] = dictupdate.update(ret['changes'], _ret['changes'])
        ret['comment'] = ' '.join([ret['comment'], _ret['comment']])
        if not _ret['result']:
            ret['result'] = _ret['result']
            if ret['result'] is False:
                return ret
    _ret = _policies_present(name, _policies, region, key, keyid, profile,
                             delete_policies)
    ret['changes'] = dictupdate.update(ret['changes'], _ret['changes'])
    ret['comment'] = ' '.join([ret['comment'], _ret['comment']])
    if not _ret['result']:
        ret['result'] = _ret['result']
    _ret = _policies_attached(name, managed_policies, region, key, keyid, profile)
    ret['changes'] = dictupdate.update(ret['changes'], _ret['changes'])
    ret['comment'] = ' '.join([ret['comment'], _ret['comment']])
    if not _ret['result']:
        ret['result'] = _ret['result']
    return ret


def _role_present(
        name,
        policy_document=None,
        path=None,
        region=None,
        key=None,
        keyid=None,
        profile=None):
    ret = {'result': True, 'comment': '', 'changes': {}}
    role = __salt__['boto_iam.describe_role'](name, region, key, keyid,
                                              profile)
    if not role:
        if __opts__['test']:
            ret['comment'] = 'IAM role {0} is set to be created.'.format(name)
            ret['result'] = None
            return ret
        created = __salt__['boto_iam.create_role'](name, policy_document,
                                                   path, region, key,
                                                   keyid, profile)
        if created:
            ret['changes']['old'] = {'role': None}
            ret['changes']['new'] = {'role': name}
            ret['comment'] = 'IAM role {0} created.'.format(name)
        else:
            ret['result'] = False
            ret['comment'] = 'Failed to create {0} IAM role.'.format(name)
    else:
        ret['comment'] = '{0} role present.'.format(name)
        update_needed = False
        _policy_document = None
        if not policy_document:
            policy = __salt__['boto_iam.build_policy'](region, key, keyid,
                                                       profile)
            if role['assume_role_policy_document'] != policy:
                update_needed = True
                _policy_document = policy
        else:
            if role['assume_role_policy_document'] != policy_document:
                update_needed = True
                _policy_document = policy_document
        if update_needed:
            if __opts__['test']:
                msg = 'Assume role policy document to be updated.'
                ret['comment'] = '{0} {1}'.format(ret['comment'], msg)
                ret['result'] = None
                return ret
            updated = __salt__['boto_iam.update_assume_role_policy'](
                name, _policy_document, region, key, keyid, profile
            )
            if updated:
                msg = 'Assume role policy document updated.'
                ret['comment'] = '{0} {1}'.format(ret['comment'], msg)
                ret['changes']['old'] = {'policy_document': role['assume_role_policy_document']}
                ret['changes']['new'] = {'policy_document': _policy_document}
            else:
                ret['result'] = False
                msg = 'Failed to update assume role policy.'
                ret['comment'] = '{0} {1}'.format(ret['comment'], msg)
    return ret


def _instance_profile_present(
        name,
        region=None,
        key=None,
        keyid=None,
        profile=None):
    ret = {'result': True, 'comment': '', 'changes': {}}
    exists = __salt__['boto_iam.instance_profile_exists'](name, region, key,
                                                          keyid, profile)
    if not exists:
        if __opts__['test']:
            msg = 'Instance profile {0} is set to be created.'
            ret['comment'] = msg.format(name)
            ret['result'] = None
            return ret
        created = __salt__['boto_iam.create_instance_profile'](name, region,
                                                               key, keyid,
                                                               profile)
        if created:
            ret['changes']['old'] = {'instance_profile': None}
            ret['changes']['new'] = {'instance_profile': name}
            ret['comment'] = 'Instance profile {0} created.'.format(name)
        else:
            ret['result'] = False
            msg = 'Failed to create {0} instance profile.'.format(name)
            ret['comment'] = msg
    return ret


def _instance_profile_associated(
        name,
        region=None,
        key=None,
        keyid=None,
        profile=None):
    ret = {'result': True, 'comment': '', 'changes': {}}
    is_associated = __salt__['boto_iam.profile_associated'](name, name, region,
                                                            key, keyid,
                                                            profile)
    if not is_associated:
        if __opts__['test']:
            msg = 'Instance profile {0} is set to be associated.'
            ret['comment'] = msg.format(name)
            ret['result'] = None
            return ret
        associated = __salt__['boto_iam.associate_profile_to_role'](name, name,
                                                                    region,
                                                                    key, keyid,
                                                                    profile)
        if associated:
            ret['changes']['old'] = {'profile_associated': None}
            ret['changes']['new'] = {'profile_associated': True}
            ret['comment'] = 'Instance profile {0} associated.'.format(name)
        else:
            ret['result'] = False
            msg = 'Failed to associate {0} instance profile with {0} role.'
            ret['comment'] = msg.format(name)
    return ret


def _policies_present(
        name,
        policies=None,
        region=None,
        key=None,
        keyid=None,
        profile=None,
        delete_policies=True):
    ret = {'result': True, 'comment': '', 'changes': {}}
    policies_to_create = {}
    policies_to_delete = []
    for policy_name, policy in six.iteritems(policies):
        _policy = __salt__['boto_iam.get_role_policy'](name, policy_name,
                                                       region, key, keyid,
                                                       profile)
        if _policy != policy:
            policies_to_create[policy_name] = policy
    _list = __salt__['boto_iam.list_role_policies'](name, region, key, keyid,
                                                    profile)
    for policy_name in _list:
        if delete_policies and policy_name not in policies:
            policies_to_delete.append(policy_name)
    if policies_to_create or policies_to_delete:
        _to_modify = list(policies_to_delete)
        _to_modify.extend(policies_to_create)
        if __opts__['test']:
            msg = '{0} policies to be modified on role {1}.'
            ret['comment'] = msg.format(', '.join(_to_modify), name)
            ret['result'] = None
            return ret
        ret['changes']['old'] = {'policies': _list}
        for policy_name, policy in six.iteritems(policies_to_create):
            policy_set = __salt__['boto_iam.create_role_policy'](name,
                                                                 policy_name,
                                                                 policy,
                                                                 region, key,
                                                                 keyid,
                                                                 profile)
            if not policy_set:
                _list = __salt__['boto_iam.list_role_policies'](name, region,
                                                                key, keyid,
                                                                profile)
                ret['changes']['new'] = {'policies': _list}
                ret['result'] = False
                msg = 'Failed to add policy {0} to role {1}'
                ret['comment'] = msg.format(policy_name, name)
                return ret
        for policy_name in policies_to_delete:
            policy_unset = __salt__['boto_iam.delete_role_policy'](name,
                                                                   policy_name,
                                                                   region, key,
                                                                   keyid,
                                                                   profile)
            if not policy_unset:
                _list = __salt__['boto_iam.list_role_policies'](name, region,
                                                                key, keyid,
                                                                profile)
                ret['changes']['new'] = {'policies': _list}
                ret['result'] = False
                msg = 'Failed to remove policy {0} from role {1}'
                ret['comment'] = msg.format(policy_name, name)
                return ret
        _list = __salt__['boto_iam.list_role_policies'](name, region, key,
                                                        keyid, profile)
        ret['changes']['new'] = {'policies': _list}
        msg = '{0} policies modified on role {1}.'
        ret['comment'] = msg.format(', '.join(_list), name)
    return ret


def _policies_attached(
        name,
        managed_policies=None,
        region=None,
        key=None,
        keyid=None,
        profile=None):
    ret = {'result': True, 'comment': '', 'changes': {}}
    policies_to_attach = []
    policies_to_detach = []
    for policy in managed_policies or []:
        entities = __salt__['boto_iam.list_entities_for_policy'](policy,
                                       entity_filter='Role',
                                       region=region, key=key, keyid=keyid,
                                       profile=profile)
        found = False
        for roledict in entities.get('policy_roles', []):
            if name == roledict.get('role_name'):
                found = True
                break
        if not found:
            policies_to_attach.append(policy)
    _list = __salt__['boto_iam.list_attached_role_policies'](name, region=region, key=key, keyid=keyid,
                                                    profile=profile)
    oldpolicies = [x.get('policy_arn') for x in _list]
    for policy_data in _list:
        if policy_data.get('policy_name') not in managed_policies \
                  and policy_data.get('policy_arn') not in managed_policies:
            policies_to_detach.append(policy_data.get('policy_arn'))
    if policies_to_attach or policies_to_detach:
        _to_modify = list(policies_to_detach)
        _to_modify.extend(policies_to_attach)
        if __opts__['test']:
            msg = '{0} policies to be modified on role {1}.'
            ret['comment'] = msg.format(', '.join(_to_modify), name)
            ret['result'] = None
            return ret
        ret['changes']['old'] = {'managed_policies': oldpolicies}
        for policy_name in policies_to_attach:
            policy_set = __salt__['boto_iam.attach_role_policy'](policy_name,
                                                                 role_name=name,
                                                                 region=region,
                                                                 key=key,
                                                                 keyid=keyid,
                                                                 profile=profile)
            if not policy_set:
                _list = __salt__['boto_iam.list_attached_role_policies'](name, region=region,
                                                                key=key,
                                                                keyid=keyid,
                                                                profile=profile)
                newpolicies = [x.get('policy_arn') for x in _list]
                ret['changes']['new'] = {'managed_policies': newpolicies}
                ret['result'] = False
                msg = 'Failed to add policy {0} to role {1}'
                ret['comment'] = msg.format(policy_name, name)
                return ret
        for policy_name in policies_to_detach:
            policy_unset = __salt__['boto_iam.detach_role_policy'](policy_name,
                                                                   role_name=name,
                                                                   region=region,
                                                                   key=key,
                                                                   keyid=keyid,
                                                                   profile=profile)
            if not policy_unset:
                _list = __salt__['boto_iam.list_attached_role_policies'](name, region=region,
                                                                key=key,
                                                                keyid=keyid,
                                                                profile=profile)
                newpolicies = [x.get('policy_arn') for x in _list]
                ret['changes']['new'] = {'managed_policies': newpolicies}
                ret['result'] = False
                msg = 'Failed to remove policy {0} from role {1}'
                ret['comment'] = msg.format(policy_name, name)
                return ret
        _list = __salt__['boto_iam.list_attached_role_policies'](name, region=region, key=key,
                                                        keyid=keyid,
                                                        profile=profile)
        newpolicies = [x.get('policy_arn') for x in _list]
        log.debug(newpolicies)
        ret['changes']['new'] = {'managed_policies': newpolicies}
        msg = '{0} policies modified on role {1}.'
        ret['comment'] = msg.format(', '.join(newpolicies), name)
    return ret


def absent(
        name,
        region=None,
        key=None,
        keyid=None,
        profile=None):
    '''
    Ensure the IAM role is deleted.

    name
        Name of the IAM role.

    region
        Region to connect to.

    key
        Secret key to be used.

    keyid
        Access key to be used.

    profile
        A dict with region, key and keyid, or a pillar key (string)
        that contains a dict with region, key and keyid.
    '''
    ret = {'name': name, 'result': True, 'comment': '', 'changes': {}}
    _ret = _policies_absent(name, region, key, keyid, profile)
    ret['changes'] = _ret['changes']
    ret['comment'] = ' '.join([ret['comment'], _ret['comment']])
    if not _ret['result']:
        ret['result'] = _ret['result']
        if ret['result'] is False:
            return ret
    _ret = _policies_detached(name, region, key, keyid, profile)
    ret['changes'] = _ret['changes']
    ret['comment'] = ' '.join([ret['comment'], _ret['comment']])
    if not _ret['result']:
        ret['result'] = _ret['result']
        if ret['result'] is False:
            return ret
    _ret = _instance_profile_disassociated(name, region, key, keyid, profile)
    ret['changes'] = dictupdate.update(ret['changes'], _ret['changes'])
    ret['comment'] = ' '.join([ret['comment'], _ret['comment']])
    if not _ret['result']:
        ret['result'] = _ret['result']
        if ret['result'] is False:
            return ret
    _ret = _instance_profile_absent(name, region, key, keyid, profile)
    ret['changes'] = dictupdate.update(ret['changes'], _ret['changes'])
    ret['comment'] = ' '.join([ret['comment'], _ret['comment']])
    if not _ret['result']:
        ret['result'] = _ret['result']
        if ret['result'] is False:
            return ret
    _ret = _role_absent(name, region, key, keyid, profile)
    ret['changes'] = dictupdate.update(ret['changes'], _ret['changes'])
    ret['comment'] = ' '.join([ret['comment'], _ret['comment']])
    if not _ret['result']:
        ret['result'] = _ret['result']
    return ret


def _role_absent(
        name,
        region=None,
        key=None,
        keyid=None,
        profile=None):
    ret = {'result': True, 'comment': '', 'changes': {}}

    exists = __salt__['boto_iam.role_exists'](name, region, key, keyid,
                                              profile)
    if exists:
        if __opts__['test']:
            ret['comment'] = 'IAM role {0} is set to be removed.'.format(
                name)
            ret['result'] = None
            return ret
        deleted = __salt__['boto_iam.delete_role'](name, region, key, keyid,
                                                   profile)
        if deleted:
            ret['changes']['old'] = {'role': name}
            ret['changes']['new'] = {'role': None}
            ret['comment'] = 'IAM role {0} removed.'.format(name)
        else:
            ret['result'] = False
            ret['comment'] = 'Failed to delete {0} iam role.'.format(name)
    else:
        ret['comment'] = '{0} role does not exist.'.format(name)
    return ret


def _instance_profile_absent(
        name,
        region=None,
        key=None,
        keyid=None,
        profile=None):
    ret = {'result': True, 'comment': '', 'changes': {}}

    exists = __salt__['boto_iam.instance_profile_exists'](name, region, key,
                                                          keyid, profile)
    if exists:
        if __opts__['test']:
            msg = 'Instance profile {0} is set to be removed.'
            ret['comment'] = msg.format(name)
            ret['result'] = None
            return ret
        deleted = __salt__['boto_iam.delete_instance_profile'](name, region,
                                                               key, keyid,
                                                               profile)
        if deleted:
            ret['changes']['old'] = {'instance_profile': name}
            ret['changes']['new'] = {'instance_profile': None}
            ret['comment'] = 'Instance profile {0} removed.'.format(name)
        else:
            ret['result'] = False
            msg = 'Failed to delete {0} instance profile.'.format(name)
            ret['comment'] = msg
    else:
        ret['comment'] = '{0} instance profile does not exist.'.format(name)
    return ret


def _policies_absent(
        name,
        region=None,
        key=None,
        keyid=None,
        profile=None):
    ret = {'result': True, 'comment': '', 'changes': {}}
    _list = __salt__['boto_iam.list_role_policies'](name, region, key, keyid,
                                                    profile)
    if not _list:
        msg = 'No policies in role {0}.'.format(name)
        ret['comment'] = msg
        return ret
    if __opts__['test']:
        msg = '{0} policies to be removed from role {1}.'
        ret['comment'] = msg.format(', '.join(_list), name)
        ret['result'] = None
        return ret
    ret['changes']['old'] = {'policies': _list}
    for policy_name in _list:
        policy_unset = __salt__['boto_iam.delete_role_policy'](name,
                                                               policy_name,
                                                               region, key,
                                                               keyid,
                                                               profile)
        if not policy_unset:
            _list = __salt__['boto_iam.list_role_policies'](name, region,
                                                            key, keyid,
                                                            profile)
            ret['changes']['new'] = {'policies': _list}
            ret['result'] = False
            msg = 'Failed to add policy {0} to role {1}'
            ret['comment'] = msg.format(policy_name, name)
            return ret
    _list = __salt__['boto_iam.list_role_policies'](name, region, key,
                                                    keyid, profile)
    ret['changes']['new'] = {'policies': _list}
    msg = '{0} policies removed from role {1}.'
    ret['comment'] = msg.format(', '.join(_list), name)
    return ret


def _policies_detached(
        name,
        region=None,
        key=None,
        keyid=None,
        profile=None):
    ret = {'result': True, 'comment': '', 'changes': {}}
    _list = __salt__['boto_iam.list_attached_role_policies'](role_name=name,
                        region=region, key=key, keyid=keyid, profile=profile)
    oldpolicies = [x.get('policy_arn') for x in _list]
    if not _list:
        msg = 'No attached policies in role {0}.'.format(name)
        ret['comment'] = msg
        return ret
    if __opts__['test']:
        msg = '{0} policies to be detached from role {1}.'
        ret['comment'] = msg.format(', '.join(oldpolicies), name)
        ret['result'] = None
        return ret
    ret['changes']['old'] = {'managed_policies': oldpolicies}
    for policy_arn in oldpolicies:
        policy_unset = __salt__['boto_iam.detach_role_policy'](policy_arn,
                                                               name,
                                                               region=region,
                                                               key=key,
                                                               keyid=keyid,
                                                               profile=profile)
        if not policy_unset:
            _list = __salt__['boto_iam.list_attached_role_policies'](name, region=region,
                                                            key=key, keyid=keyid,
                                                            profile=profile)
            newpolicies = [x.get('policy_arn') for x in _list]
            ret['changes']['new'] = {'managed_policies': newpolicies}
            ret['result'] = False
            msg = 'Failed to detach {0} from role {1}'
            ret['comment'] = msg.format(policy_arn, name)
            return ret
    _list = __salt__['boto_iam.list_attached_role_policies'](name, region=region, key=key,
                                                    keyid=keyid, profile=profile)
    newpolicies = [x.get('policy_arn') for x in _list]
    ret['changes']['new'] = {'managed_policies': newpolicies}
    msg = '{0} policies detached from role {1}.'
    ret['comment'] = msg.format(', '.join(newpolicies), name)
    return ret


def _instance_profile_disassociated(
        name,
        region=None,
        key=None,
        keyid=None,
        profile=None):
    ret = {'result': True, 'comment': '', 'changes': {}}
    is_associated = __salt__['boto_iam.profile_associated'](name, name, region,
                                                            key, keyid,
                                                            profile)
    if is_associated:
        if __opts__['test']:
            msg = 'Instance profile {0} is set to be disassociated.'
            ret['comment'] = msg.format(name)
            ret['result'] = None
            return ret
        associated = __salt__['boto_iam.disassociate_profile_from_role'](name, name, region, key, keyid, profile)
        if associated:
            ret['changes']['old'] = {'profile_associated': True}
            ret['changes']['new'] = {'profile_associated': False}
            msg = 'Instance profile {0} disassociated.'.format(name)
            ret['comment'] = msg
        else:
            ret['result'] = False
            msg = 'Failed to disassociate {0} instance profile from {0} role.'
            ret['comment'] = msg.format(name)
    return ret



# -*- coding: utf-8 -*-
'''
Network tools to run from the Master
'''

# Import python libs
from __future__ import print_function
from __future__ import absolute_import
import socket

# Import salt libs
import salt.utils


def wollist(maclist, bcast='255.255.255.255', destport=9):
    '''
    Send a "Magic Packet" to wake up a list of Minions.
    This list must contain one MAC hardware address per line

    CLI Example:

    .. code-block:: bash

        salt-run network.wollist '/path/to/maclist'
        salt-run network.wollist '/path/to/maclist' 255.255.255.255 7
        salt-run network.wollist '/path/to/maclist' 255.255.255.255 7
    '''
    ret = []
    try:
        with salt.utils.fopen(maclist, 'r') as ifile:
            for mac in ifile:
                wol(mac.strip(), bcast, destport)
                print('Waking up {0}'.format(mac.strip()))
                ret.append(mac)
    except Exception as err:
        __jid_event__.fire_event({'error': 'Failed to open the MAC file. Error: {0}'.format(err)}, 'progress')
        return []
    return ret


def wol(mac, bcast='255.255.255.255', destport=9):
    '''
    Send a "Magic Packet" to wake up a Minion

    CLI Example:

    .. code-block:: bash

        salt-run network.wol 08-00-27-13-69-77
        salt-run network.wol 080027136977 255.255.255.255 7
        salt-run network.wol 08:00:27:13:69:77 255.255.255.255 7
    '''
    dest = salt.utils.mac_str_to_bytes(mac)
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
    sock.sendto(b'\xff' * 6 + dest * 16, (bcast, int(destport)))
    return True



# -*- coding: utf-8 -*-
'''
Tests for the archive state
'''
# Import python libs
from __future__ import absolute_import
import errno
import logging
import os
import platform
import socket
import threading
import tornado.httpserver
import tornado.ioloop
import tornado.web

# Import Salt Testing libs
from salttesting import skipIf
from salttesting.helpers import ensure_in_syspath
ensure_in_syspath('../../')

# Import salt libs
import integration
import salt.utils

# Setup logging
log = logging.getLogger(__name__)

STATE_DIR = os.path.join(integration.FILES, 'file', 'base')
if salt.utils.is_windows():
    ARCHIVE_DIR = os.path.join("c:/", "tmp")
else:
    ARCHIVE_DIR = '/tmp/archive'

PORT = 9999
ARCHIVE_TAR_SOURCE = 'http://localhost:{0}/custom.tar.gz'.format(PORT)
UNTAR_FILE = os.path.join(ARCHIVE_DIR, 'custom/README')
ARCHIVE_TAR_HASH = 'md5=7643861ac07c30fe7d2310e9f25ca514'

REDHAT7 = False
QUERY_OS = platform.dist()
OS_VERSION = QUERY_OS[1]
OS_FAMILY = QUERY_OS[0]
if '7' in OS_VERSION and 'centos' in OS_FAMILY:
    REDHAT7 = True


@skipIf(not REDHAT7, 'Only run on redhat7 for now due to hanging issues on other OS')
class ArchiveTest(integration.ModuleCase,
                  integration.SaltReturnAssertsMixIn):
    '''
    Validate the archive state
    '''
    @classmethod
    def webserver(cls):
        '''
        method to start tornado
        static web app
        '''
        application = tornado.web.Application([(r"/(.*)", tornado.web.StaticFileHandler,
                                              {"path": STATE_DIR})])
        cls.server = tornado.httpserver.HTTPServer(application)
        cls.server.listen(PORT)
        tornado.ioloop.IOLoop.instance().start()

    @classmethod
    def setUpClass(cls):
        '''
        start tornado app on thread
        and wait till its running
        '''
        cls.server_thread = threading.Thread(target=cls.webserver)
        cls.server_thread.daemon = True
        cls.server_thread.start()
        # check if tornado app is up
        port_closed = True
        while port_closed:
            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            result = sock.connect_ex(('127.0.0.1', PORT))
            if result == 0:
                port_closed = False

    @classmethod
    def tearDownClass(cls):
        tornado.ioloop.IOLoop.instance().stop()
        cls.server_thread.join()
        cls.server.stop()

    def setUp(self):
        self._clear_archive_dir()

    def tearDown(self):
        self._clear_archive_dir()

    @staticmethod
    def _clear_archive_dir():
        try:
            salt.utils.rm_rf(ARCHIVE_DIR)
        except OSError as exc:
            if exc.errno != errno.ENOENT:
                raise

    def _check_extracted(self, path):
        '''
        function to check if file was extracted
        '''
        log.debug('Checking for extracted file: %s', path)
        self.assertTrue(os.path.isfile(path))

    def test_archive_extracted_skip_verify(self):
        '''
        test archive.extracted with skip_verify
        '''
        ret = self.run_state('archive.extracted', name=ARCHIVE_DIR,
                             source=ARCHIVE_TAR_SOURCE, archive_format='tar',
                             skip_verify=True)
        log.debug('ret = %s', ret)
        if 'Timeout' in ret:
            self.skipTest('Timeout talking to local tornado server.')
        self.assertSaltTrueReturn(ret)

        self._check_extracted(UNTAR_FILE)

    def test_archive_extracted_with_source_hash(self):
        '''
        test archive.extracted without skip_verify
        only external resources work to check to
        ensure source_hash is verified correctly
        '''
        ret = self.run_state('archive.extracted', name=ARCHIVE_DIR,
                             source=ARCHIVE_TAR_SOURCE, archive_format='tar',
                             source_hash=ARCHIVE_TAR_HASH)
        log.debug('ret = %s', ret)
        if 'Timeout' in ret:
            self.skipTest('Timeout talking to local tornado server.')

        self.assertSaltTrueReturn(ret)

        self._check_extracted(UNTAR_FILE)

    @skipIf(os.geteuid() != 0, 'you must be root to run this test')
    def test_archive_extracted_with_root_user_and_group(self):
        '''
        test archive.extracted with user and group set to "root"
        '''
        ret = self.run_state('archive.extracted', name=ARCHIVE_DIR,
                             source=ARCHIVE_TAR_SOURCE, archive_format='tar',
                             source_hash=ARCHIVE_TAR_HASH,
                             user='root', group='root')
        log.debug('ret = %s', ret)
        if 'Timeout' in ret:
            self.skipTest('Timeout talking to local tornado server.')

        self.assertSaltTrueReturn(ret)

        self._check_extracted(UNTAR_FILE)

    @skipIf(os.geteuid() != 0, 'you must be root to run this test')
    def test_archive_extracted_with_strip_in_options(self):
        '''
        test archive.extracted with --strip in options
        '''
        ret = self.run_state('archive.extracted', name=ARCHIVE_DIR,
                             source=ARCHIVE_TAR_SOURCE,
                             source_hash=ARCHIVE_TAR_HASH,
                             options='--strip=1',
                             enforce_toplevel=False)
        log.debug('ret = %s', ret)
        if 'Timeout' in ret:
            self.skipTest('Timeout talking to local tornado server.')

        self.assertSaltTrueReturn(ret)

        self._check_extracted(os.path.join(ARCHIVE_DIR, 'README'))

    @skipIf(os.geteuid() != 0, 'you must be root to run this test')
    def test_archive_extracted_with_strip_components_in_options(self):
        '''
        test archive.extracted with --strip-components in options
        '''
        ret = self.run_state('archive.extracted', name=ARCHIVE_DIR,
                             source=ARCHIVE_TAR_SOURCE,
                             source_hash=ARCHIVE_TAR_HASH,
                             options='--strip-components=1',
                             enforce_toplevel=False)
        log.debug('ret = %s', ret)
        if 'Timeout' in ret:
            self.skipTest('Timeout talking to local tornado server.')

        self.assertSaltTrueReturn(ret)

        self._check_extracted(os.path.join(ARCHIVE_DIR, 'README'))

    def test_archive_extracted_without_archive_format(self):
        '''
        test archive.extracted with no archive_format option
        '''
        ret = self.run_state('archive.extracted', name=ARCHIVE_DIR,
                             source=ARCHIVE_TAR_SOURCE,
                             source_hash=ARCHIVE_TAR_HASH)
        log.debug('ret = %s', ret)
        if 'Timeout' in ret:
            self.skipTest('Timeout talking to local tornado server.')
        self.assertSaltTrueReturn(ret)

        self._check_extracted(UNTAR_FILE)

    def test_archive_extracted_with_cmd_unzip_false(self):
        '''
        test archive.extracted using use_cmd_unzip argument as false
        '''

        ret = self.run_state('archive.extracted', name=ARCHIVE_DIR,
                             source=ARCHIVE_TAR_SOURCE,
                             source_hash=ARCHIVE_TAR_HASH,
                             use_cmd_unzip=False,
                             archive_format='tar')
        log.debug('ret = %s', ret)
        if 'Timeout' in ret:
            self.skipTest('Timeout talking to local tornado server.')
        self.assertSaltTrueReturn(ret)

        self._check_extracted(UNTAR_FILE)


if __name__ == '__main__':
    from integration import run_tests
    run_tests(ArchiveTest)



# -*- coding: utf-8 -*-

# Import python libs
from __future__ import absolute_import
import os
import yaml
import shutil
import time

# Import Salt Testing libs
from salttesting.helpers import ensure_in_syspath
ensure_in_syspath('../../')

# Import salt libs
import integration
import salt.utils


def minion_in_returns(minion, lines):
    return bool([True for line in lines if line == '{0}:'.format(minion)])


class MatchTest(integration.ShellCase, integration.ShellCaseCommonTestsMixIn):
    '''
    Test salt matchers
    '''
    _call_binary_ = 'salt'

    def test_list(self):
        '''
        test salt -L matcher
        '''
        data = self.run_salt('-L minion test.ping')
        data = '\n'.join(data)
        self.assertIn('minion', data)
        self.assertNotIn('sub_minion', data)
        data = self.run_salt('-L minion,sub_minion test.ping')
        data = '\n'.join(data)
        self.assertIn('minion', data)
        self.assertIn('sub_minion', data)

    def test_compound(self):
        '''
        test salt compound matcher
        '''
        data = self.run_salt('-C "min* and G@test_grain:cheese" test.ping')
        self.assertTrue(minion_in_returns('minion', data))
        self.assertFalse(minion_in_returns('sub_minion', data))
        time.sleep(2)
        data = self.run_salt('-C "min* and not G@test_grain:foo" test.ping')
        self.assertTrue(minion_in_returns('minion', data))
        self.assertFalse(minion_in_returns('sub_minion', data))
        time.sleep(2)
        data = self.run_salt('-C "min* not G@test_grain:foo" test.ping')
        self.assertTrue(minion_in_returns('minion', data))
        self.assertFalse(minion_in_returns('sub_minion', data))
        time.sleep(2)
        match = 'P@test_grain:^cheese$ and * and G@test_grain:cheese'
        data = self.run_salt('-t 1 -C \'{0}\' test.ping'.format(match))
        self.assertTrue(minion_in_returns('minion', data))
        self.assertFalse(minion_in_returns('sub_minion', data))
        time.sleep(2)
        match = 'L@sub_minion and E@.*'
        data = self.run_salt('-t 1 -C "{0}" test.ping'.format(match))
        self.assertTrue(minion_in_returns('sub_minion', data))
        self.assertFalse(minion_in_returns('minion', data))
        time.sleep(2)
        data = self.run_salt("-C 'not sub_minion' test.ping")
        self.assertTrue(minion_in_returns('minion', data))
        self.assertFalse(minion_in_returns('sub_minion', data))
        time.sleep(2)
        data = self.run_salt("-C '* and ( not G@test_grain:cheese )' test.ping")
        self.assertFalse(minion_in_returns('minion', data))
        self.assertTrue(minion_in_returns('sub_minion', data))
        time.sleep(2)
        data = self.run_salt("-C 'G%@planets%merc*' test.ping")
        self.assertTrue(minion_in_returns('minion', data))
        self.assertFalse(minion_in_returns('sub_minion', data))
        time.sleep(2)
        data = self.run_salt("-C 'P%@planets%^(mercury|saturn)$' test.ping")
        self.assertTrue(minion_in_returns('minion', data))
        self.assertTrue(minion_in_returns('sub_minion', data))
        time.sleep(2)
        data = self.run_salt("-C 'I%@companions%three%sarah*' test.ping")
        self.assertTrue(minion_in_returns('minion', data))
        self.assertTrue(minion_in_returns('sub_minion', data))
        time.sleep(2)
        data = self.run_salt("-C 'J%@knights%^(Lancelot|Galahad)$' test.ping")
        self.assertTrue(minion_in_returns('minion', data))
        self.assertTrue(minion_in_returns('sub_minion', data))

    def test_nodegroup(self):
        '''
        test salt nodegroup matcher
        '''
        data = self.run_salt('-N min test.ping')
        self.assertTrue(minion_in_returns('minion', data))
        self.assertFalse(minion_in_returns('sub_minion', data))
        time.sleep(2)
        data = self.run_salt('-N sub_min test.ping')
        self.assertFalse(minion_in_returns('minion', data))
        self.assertTrue(minion_in_returns('sub_minion', data))
        time.sleep(2)
        data = self.run_salt('-N mins test.ping')
        self.assertTrue(minion_in_returns('minion', data))
        self.assertTrue(minion_in_returns('sub_minion', data))
        time.sleep(2)
        data = self.run_salt('-N unknown_nodegroup test.ping')
        self.assertFalse(minion_in_returns('minion', data))
        self.assertFalse(minion_in_returns('sub_minion', data))
        time.sleep(2)
        data = self.run_salt('-N redundant_minions test.ping')
        self.assertTrue(minion_in_returns('minion', data))
        self.assertTrue(minion_in_returns('sub_minion', data))
        time.sleep(2)
        data = '\n'.join(self.run_salt('-N nodegroup_loop_a test.ping'))
        self.assertIn('No minions matched', data)
        time.sleep(2)
        data = self.run_salt("-N multiline_nodegroup test.ping")
        self.assertTrue(minion_in_returns('minion', data))
        self.assertTrue(minion_in_returns('sub_minion', data))

    def test_glob(self):
        '''
        test salt glob matcher
        '''
        data = self.run_salt('minion test.ping')
        data = '\n'.join(data)
        self.assertIn('minion', data)
        self.assertNotIn('sub_minion', data)
        data = self.run_salt('"*" test.ping')
        data = '\n'.join(data)
        self.assertIn('minion', data)
        self.assertIn('sub_minion', data)

    def test_regex(self):
        '''
        test salt regex matcher
        '''
        data = self.run_salt('-E "^minion$" test.ping')
        data = '\n'.join(data)
        self.assertIn('minion', data)
        self.assertNotIn('sub_minion', data)
        data = self.run_salt('-E ".*" test.ping')
        data = '\n'.join(data)
        self.assertIn('minion', data)
        self.assertIn('sub_minion', data)

    def test_grain(self):
        '''
        test salt grain matcher
        '''
        # Sync grains
        self.run_salt('-t1 "*" saltutil.sync_grains')
        # First-level grain (string value)
        data = self.run_salt('-t 1 -G "test_grain:cheese" test.ping')
        data = '\n'.join(data)
        self.assertIn('minion', data)
        self.assertNotIn('sub_minion', data)
        data = self.run_salt('-G "test_grain:spam" test.ping')
        data = '\n'.join(data)
        self.assertIn('sub_minion', data)
        self.assertNotIn('minion', data.replace('sub_minion', 'stub'))
        # Custom grain
        data = self.run_salt('-t 1 -G "match:maker" test.ping')
        data = '\n'.join(data)
        self.assertIn('minion', data)
        self.assertIn('sub_minion', data)
        # First-level grain (list member)
        data = self.run_salt('-t 1 -G "planets:earth" test.ping')
        data = '\n'.join(data)
        self.assertIn('minion', data)
        self.assertNotIn('sub_minion', data)
        data = self.run_salt('-G "planets:saturn" test.ping')
        data = '\n'.join(data)
        self.assertIn('sub_minion', data)
        self.assertNotIn('minion', data.replace('sub_minion', 'stub'))
        data = self.run_salt('-G "planets:pluto" test.ping')
        expect = None
        if self.master_opts['transport'] in ('zeromq', 'tcp'):
            expect = (
                'No minions matched the target. '
                'No command was sent, no jid was '
                'assigned.'
            )
        elif self.master_opts['transport'] == 'raet':
            expect = ''
        self.assertEqual(
            ''.join(data),
            expect
        )
        # Nested grain (string value)
        data = self.run_salt('-t 1 -G "level1:level2:foo" test.ping')
        data = '\n'.join(data)
        self.assertIn('minion', data)
        self.assertNotIn('sub_minion', data)
        data = self.run_salt('-G "level1:level2:bar" test.ping')
        data = '\n'.join(data)
        self.assertIn('sub_minion', data)
        self.assertNotIn('minion', data.replace('sub_minion', 'stub'))
        # Nested grain (list member)
        data = self.run_salt('-t 1 -G "companions:one:ian" test.ping')
        data = '\n'.join(data)
        self.assertIn('minion', data)
        self.assertNotIn('sub_minion', data)
        data = self.run_salt('-G "companions:two:jamie" test.ping')
        data = '\n'.join(data)
        self.assertIn('sub_minion', data)
        self.assertNotIn('minion', data.replace('sub_minion', 'stub'))
        # Test for issue: https://github.com/saltstack/salt/issues/19651
        data = self.run_salt('-G "companions:*:susan" test.ping')
        data = '\n'.join(data)
        self.assertIn('minion:', data)
        self.assertNotIn('sub_minion', data)
        # Test to ensure wildcard at end works correctly
        data = self.run_salt('-G "companions:one:*" test.ping')
        data = '\n'.join(data)
        self.assertIn('minion:', data)
        self.assertNotIn('sub_minion', data)
        # Test to ensure multiple wildcards works correctly
        data = self.run_salt('-G "companions:*:*" test.ping')
        data = '\n'.join(data)
        self.assertIn('minion:', data)
        self.assertIn('sub_minion', data)

    def test_regrain(self):
        '''
        test salt grain matcher
        '''
        data = self.run_salt(
            '-t 1 --grain-pcre "test_grain:^cheese$" test.ping'
        )
        data = '\n'.join(data)
        self.assertIn('minion', data)
        self.assertNotIn('sub_minion', data)
        data = self.run_salt('--grain-pcre "test_grain:.*am$" test.ping')
        data = '\n'.join(data)
        self.assertIn('sub_minion', data)
        self.assertNotIn('minion', data.replace('sub_minion', 'stub'))

    def test_pillar(self):
        '''
        test pillar matcher
        '''
        # First-level pillar (string value)
        data = self.run_salt('-I "monty:python" test.ping')
        data = '\n'.join(data)
        self.assertIn('minion', data)
        self.assertIn('sub_minion', data)
        # First-level pillar (string value, only in sub_minion)
        data = self.run_salt('-I "sub:sub_minion" test.ping')
        data = '\n'.join(data)
        self.assertIn('sub_minion', data)
        self.assertNotIn('minion', data.replace('sub_minion', 'stub'))
        # First-level pillar (list member)
        data = self.run_salt('-I "knights:Bedevere" test.ping')
        data = '\n'.join(data)
        self.assertIn('minion', data)
        self.assertIn('sub_minion', data)
        # Nested pillar (string value)
        data = self.run_salt('-I "level1:level2:foo" test.ping')
        data = '\n'.join(data)
        self.assertIn('minion', data)
        self.assertIn('sub_minion', data)
        # Nested pillar (list member)
        data = self.run_salt('-I "companions:three:sarah jane" test.ping')
        data = '\n'.join(data)
        self.assertIn('minion', data)
        self.assertIn('sub_minion', data)

    def test_repillar(self):
        '''
        test salt pillar PCRE matcher
        '''
        data = self.run_salt('-J "monty:^(python|hall)$" test.ping')
        data = '\n'.join(data)
        self.assertIn('minion', data)
        self.assertIn('sub_minion', data)
        data = self.run_salt('--pillar-pcre "knights:^(Robin|Lancelot)$" test.ping')
        data = '\n'.join(data)
        self.assertIn('sub_minion', data)
        self.assertIn('minion', data.replace('sub_minion', 'stub'))

    def test_ipcidr(self):
        subnets_data = self.run_salt('--out yaml \'*\' network.subnets')
        yaml_data = yaml.load('\n'.join(subnets_data))

        # We're just after the first defined subnet from 'minion'
        subnet = yaml_data['minion'][0]

        data = self.run_salt('-S {0} test.ping'.format(subnet))
        data = '\n'.join(data)
        self.assertIn('minion', data)
        self.assertIn('sub_minion', data)

    def test_static(self):
        '''
        test salt static call
        '''
        data = self.run_salt('minion test.ping --static')
        data = '\n'.join(data)
        self.assertIn('minion', data)

    def test_salt_documentation(self):
        '''
        Test to see if we're supporting --doc
        '''
        data = self.run_salt('-d "*" user')
        self.assertIn('user.add:', data)

    def test_salt_documentation_arguments_not_assumed(self):
        '''
        Test to see if we're not auto-adding '*' and 'sys.doc' to the call
        '''
        data = self.run_salt('-d -t 20')
        if data:
            self.assertIn('user.add:', data)
        data = self.run_salt('"*" -d -t 20')
        if data:
            self.assertIn('user.add:', data)
        data = self.run_salt('"*" -d user -t 20')
        self.assertIn('user.add:', data)
        data = self.run_salt('"*" sys.doc -d user -t 20')
        self.assertIn('user.add:', data)
        data = self.run_salt('"*" sys.doc user -t 20')
        self.assertIn('user.add:', data)

    def test_salt_documentation_too_many_arguments(self):
        '''
        Test to see if passing additional arguments shows an error
        '''
        data = self.run_salt('-d minion salt ldap.search "filter=ou=People"', catch_stderr=True)
        self.assertIn('You can only get documentation for one method at one time', '\n'.join(data[1]))

    def test_issue_7754(self):
        old_cwd = os.getcwd()
        config_dir = os.path.join(integration.TMP, 'issue-7754')
        if not os.path.isdir(config_dir):
            os.makedirs(config_dir)

        os.chdir(config_dir)

        config_file_name = 'master'
        with salt.utils.fopen(self.get_config_file_path(config_file_name), 'r') as fhr:
            config = yaml.load(
                fhr.read()
            )
            config['log_file'] = 'file:///dev/log/LOG_LOCAL3'
            with salt.utils.fopen(os.path.join(config_dir, config_file_name), 'w') as fhw:
                fhw.write(
                    yaml.dump(config, default_flow_style=False)
            )
        ret = self.run_script(
            self._call_binary_,
            '--config-dir {0} minion test.ping'.format(
                config_dir
            ),
            timeout=60,
            catch_stderr=True,
            with_retcode=True
        )
        try:
            self.assertIn('minion', '\n'.join(ret[0]))
            self.assertFalse(os.path.isdir(os.path.join(config_dir, 'file:')))
        except AssertionError:
            # We now fail when we're unable to properly set the syslog logger
            self.assertIn(
                'Failed to setup the Syslog logging handler', '\n'.join(ret[1])
            )
            self.assertEqual(ret[2], 2)
        finally:
            self.chdir(old_cwd)
            if os.path.isdir(config_dir):
                shutil.rmtree(config_dir)


if __name__ == '__main__':
    from integration import run_tests
    run_tests(MatchTest)



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Rupesh Tare <rupesht@saltstack.com>`
'''

# Import Python libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import TestCase, skipIf
from salttesting.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
from salt.modules import hadoop

# Globals
hadoop.__salt__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class HadoopTestCase(TestCase):
    '''
    Test cases for salt.modules.hadoop
    '''
    def test_version(self):
        '''
        Test for Return version from hadoop version
        '''
        mock = MagicMock(return_value="A \nB \n")
        with patch.dict(hadoop.__salt__, {'cmd.run': mock}):
            self.assertEqual(hadoop.version(), 'B')

    def test_dfs(self):
        '''
        Test for Execute a command on DFS
        '''
        with patch.object(hadoop, '_hadoop_cmd', return_value='A'):
            self.assertEqual(hadoop.dfs('command'), 'A')

        self.assertEqual(hadoop.dfs(), 'Error: command must be provided')

    def test_dfs_present(self):
        '''
        Test for Check if a file or directory is present on the distributed FS.
        '''
        with patch.object(hadoop, '_hadoop_cmd',
                          side_effect=['No such file or directory', 'A']):
            self.assertFalse(hadoop.dfs_present('path'))
            self.assertTrue(hadoop.dfs_present('path'))

    def test_dfs_absent(self):
        '''
        Test for Check if a file or directory is absent on the distributed FS.
        '''
        with patch.object(hadoop, '_hadoop_cmd',
                          side_effect=['No such file or directory', 'A']):
            self.assertTrue(hadoop.dfs_absent('path'))
            self.assertFalse(hadoop.dfs_absent('path'))

    def test_namenode_format(self):
        '''
        Test for Format a name node
        '''
        with patch.object(hadoop, '_hadoop_cmd', return_value='A'):
            self.assertEqual(hadoop.namenode_format('force'), 'A')


if __name__ == '__main__':
    from integration import run_tests
    run_tests(HadoopTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Jayesh Kariya <jayeshk@saltstack.com>`
'''
# Import Python libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import skipIf, TestCase
from salttesting.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    patch
)

from salttesting.helpers import ensure_in_syspath

ensure_in_syspath('../../')

# Import Salt Libs
from salt.states import serverdensity_device

serverdensity_device.__salt__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class ServerdensityDeviceTestCase(TestCase):
    '''
    Test cases for salt.states.serverdensity_device
    '''
    # 'monitored' function tests: 1

    def test_monitored(self):
        '''
        Test to device is monitored with Server Density.
        '''
        name = 'my_special_server'

        ret = {'name': name,
               'changes': {},
               'result': True,
               'comment': ''}

        mock_dict = MagicMock(return_value={'id': name})
        mock_t = MagicMock(side_effect=[True, {'agentKey': True},
                                        [{'agentKey': True}]])
        mock_sd = MagicMock(side_effect=[['sd-agent'], [], True])
        with patch.dict(serverdensity_device.__salt__,
                        {'status.all_status': mock_dict,
                         'grains.items': mock_dict,
                         'serverdensity_device.ls': mock_t,
                         'pkg.list_pkgs': mock_sd,
                         'serverdensity_device.install_agent': mock_sd}):
            comt = ('Such server name already exists in this'
                    ' Server Density account. And sd-agent is installed')
            ret.update({'comment': comt})
            self.assertDictEqual(serverdensity_device.monitored(name), ret)

            comt = ('Successfully installed agent and created'
                    ' device in Server Density db.')
            ret.update({'comment': comt, 'changes': {'created_device':
                                                     {'agentKey': True},
                                                     'installed_agent': True}})
            self.assertDictEqual(serverdensity_device.monitored(name), ret)


if __name__ == '__main__':
    from integration import run_tests
    run_tests(ServerdensityDeviceTestCase, needs_daemon=False)
